@@ -1,5 +1,8 @@
 #include "docview.h"
 #include "lvdocview.h"
+#include "lvgraydrawbuf.h"
+#include "lvcolordrawbuf.h"
+#include "lvstreamutils.h"
 //#include "crgl.h"
 
 #if defined(__arm__) || defined(__aarch64__) || defined(__i386__) || defined(__mips__)
@@ -67,7 +70,7 @@ public:
 		_docview->setCallback( _oldcallback );
 	}
     /// on starting file loading
-    virtual void OnLoadFileStart( lString16 filename )
+    virtual void OnLoadFileStart( lString32 filename )
     {
 		CRLog::info("DocViewCallback::OnLoadFileStart() called");
     	_env->CallVoidMethod(_obj, _OnLoadFileStart, _env.toJavaString(filename));
@@ -79,7 +82,7 @@ public:
     	jobject e = _env.enumByNativeId("org/coolreader/crengine/DocumentFormat", (int)fileFormat);
     	jstring css = (jstring)_env->CallObjectMethod(_obj, _OnLoadFileFormatDetected, e);
     	if ( css ) {
-    		lString16 s = _env.fromJavaString(css);
+    		lString32 s = _env.fromJavaString(css);
     		CRLog::info("OnLoadFileFormatDetected: setting CSS for format %d", (int)fileFormat);
     		_docview->setStyleSheet( UnicodeToUtf8(s) );
     	}
@@ -127,16 +130,16 @@ public:
     	jboolean res = _env->CallBooleanMethod(_obj, _OnExportProgress, (jint)(percent*100));
     }
     /// file load finiished with error
-    virtual void OnLoadFileError( lString16 message )
+    virtual void OnLoadFileError( lString32 message )
     {
 		CRLog::info("DocViewCallback::OnLoadFileError() called");
     	_env->CallVoidMethod(_obj, _OnLoadFileError, _env.toJavaString(message));
     }
     /// Override to handle external links
-    virtual void OnExternalLink( lString16 url, ldomNode * node )
+    virtual void OnExternalLink( lString32 url, ldomNode * node )
     {
 		CRLog::info("DocViewCallback::OnExternalLink() called");
-    	lString16 path = ldomXPointer(node,0).toString();
+    	lString32 path = ldomXPointer(node,0).toString();
     	_env->CallVoidMethod(_obj, _OnExternalLink, _env.toJavaString(url), _env.toJavaString(path));
     }
     virtual void OnImageCacheClear()
@@ -155,11 +158,13 @@ public:
 
 CRTimerUtil _timeoutControl;
 
+#if USE_LIMITED_FONT_SIZES_SET
 #define DECL_DEF_CR_FONT_SIZES static int cr_font_sizes[] = \
  { 9, 10, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, \
    31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 42, 44, 48, 52, 56, 60, 64, 68, 72, 78, 84, 90, 110, 130, 150, 170, 200, 230, 260, 300, 340 }
 
 DECL_DEF_CR_FONT_SIZES;
+#endif
 
 #define NO_BATTERY_GAUGE 1
 
@@ -838,13 +843,18 @@ DocViewNative::DocViewNative()
     getBatteryIcons(icons, _batteryIconColor, _batteryIconSize);
     _docview->setBatteryIcons(icons);
 
+#if USE_LIMITED_FONT_SIZES_SET
     LVArray<int> sizes( cr_font_sizes, sizeof(cr_font_sizes)/sizeof(int) );
-    _docview->setShowCover( true );
     _docview->setFontSizes( sizes, true );
+#else
+    _docview->setMinFontSize(9);
+    _docview->setMaxFontSize(340);
+#endif
+	_docview->setShowCover( true );
 	_docview->setFontSize(24);
 	_docview->setBatteryFont( fontMan->GetFont( 16, 600, false, css_ff_sans_serif, lString8("Droid Sans") ));
 	
-	_docview->createDefaultDocument(lString16("Welcome to CoolReader"), lString16("Please select file to open"));
+	_docview->createDefaultDocument(lString32("Welcome to CoolReader"), lString32("Please select file to open"));
 }
 
 DocViewNative::~DocViewNative()
@@ -867,12 +877,12 @@ static DocViewNative * getNative(JNIEnv * env, jobject _this)
 	return res;
 }
 
-void DocViewNative::createDefaultDocument( lString16 title, lString16 message )
+void DocViewNative::createDefaultDocument( const lString32& title, const lString32& message )
 {
 	_docview->createDefaultDocument(title, message);
 }
 
-bool DocViewNative::loadDocument( lString16 filename )
+bool DocViewNative::loadDocument( const lString32& filename )
 {
 	CRLog::info("Loading document %s", LCSTR(filename));
 	bool res = _docview->LoadDocument(filename.c_str());
@@ -885,12 +895,31 @@ bool DocViewNative::loadDocument( lString16 filename )
 			// 1. I/O error - failed to open file
 			// 2. open archive without supported files
 			CRLog::error("Document is NULL, inserting stub.");
-			_docview->createDefaultDocument(lString16::empty_str, Utf8ToUnicode("Error while opening file!"));
+			_docview->createDefaultDocument(lString32::empty_str, Utf8ToUnicode("Error while opening file!"));
 		}
 	}
     return res;
 }
 
+bool DocViewNative::loadDocument( LVStreamRef stream, const lString32& contentPath )
+{
+	CRLog::info("Loading document from memory stream, content path: %s", LCSTR(contentPath));
+	bool res = _docview->LoadDocument(stream, contentPath.c_str(), false);
+	if (res)
+		CRLog::info("Document %s is loaded successfully", LCSTR(contentPath));
+	else {
+		CRLog::info("Document %s not is loaded due to error", LCSTR(contentPath));
+		if (_docview->getDocument() == NULL) {
+			// _docview->LoadDocument() can return false with _docview->m_doc == NULL when:
+			// 1. I/O error - failed to open file
+			// 2. open archive without supported files
+			CRLog::error("Document is NULL, inserting stub.");
+			_docview->createDefaultDocument(lString32::empty_str, Utf8ToUnicode("Error while opening file!"));
+		}
+	}
+	return res;
+}
+
 bool DocViewNative::openRecentBook()
 {
 	CRLog::debug("DocViewNative::openRecentBook()");
@@ -907,14 +936,14 @@ bool DocViewNative::openRecentBook()
     CRLog::info("DocViewNative::openRecentBook() : %d files found in history, startIndex=%d", files.length(), index);
     if ( index < files.length() ) {
         CRFileHistRecord * file = files.get( index );
-        lString16 fn = file->getFilePathName();
+        lString32 fn = file->getFilePathName();
         CRLog::info("DocViewNative::openRecentBook() : checking file %s", LCSTR(fn));
         // TODO: check error
         if ( LVFileExists(fn) ) {
             bool res = loadDocument( fn );
             if (!res && _docview->getDocument() == NULL) {
                 CRLog::error("Document is NULL, inserting stub.");
-                _docview->createDefaultDocument(lString16::empty_str, Utf8ToUnicode("Error while opening file!"));
+                _docview->createDefaultDocument(lString32::empty_str, Utf8ToUnicode("Error while opening file!"));
             }
             return res;
         } else {
@@ -934,7 +963,7 @@ bool DocViewNative::closeBook()
 	if ( _docview->isDocumentOpened() ) {
 	    _docview->savePosition();
         _docview->getDocument()->updateMap();
-        saveHistory(lString16::empty_str);
+        saveHistory(lString32::empty_str);
 	    _docview->close();
 	    return true;
 	}
@@ -946,7 +975,7 @@ void DocViewNative::clearSelection()
     _docview->clearSelection();
 }
 
-bool DocViewNative::findText( lString16 pattern, int origin, bool reverse, bool caseInsensitive )
+bool DocViewNative::findText( lString32 pattern, int origin, bool reverse, bool caseInsensitive )
 {
     if ( pattern.empty() )
         return false;
@@ -1005,7 +1034,7 @@ bool DocViewNative::findText( lString16 pattern, int origin, bool reverse, bool
 
 
 
-bool DocViewNative::loadHistory( lString16 filename )
+bool DocViewNative::loadHistory( lString32 filename )
 {
     CRFileHist * hist = _docview->getHistory();
 	if ( !filename.empty() )
@@ -1029,7 +1058,7 @@ bool DocViewNative::loadHistory( lString16 filename )
     return res;
 }
 
-bool DocViewNative::saveHistory( lString16 filename )
+bool DocViewNative::saveHistory( lString32 filename )
 {
 	if ( !filename.empty() )
 		historyFileName = filename;
@@ -1077,6 +1106,179 @@ int DocViewNative::doCommand( int cmd, int param )
    	return 1;
 }
 
+lString32 DocViewNative::getLink( int x, int y, int r )
+{
+	int step = 5;
+	int n = r / step;
+	r = n * step;
+	if ( r==0 )
+		return getLink(x, y);
+	lString32 link;
+	for ( int xx = -r; xx<=r; xx+=step ) {
+		link = getLink( x+xx, y-r );
+		if ( !link.empty() )
+			return link;
+		link = getLink( x+xx, y+r );
+		if ( !link.empty() )
+			return link;
+	}
+	for ( int yy = -r + step; yy<=r - step; yy+=step ) {
+		link = getLink( x+r, y+yy );
+		if ( !link.empty() )
+			return link;
+		link = getLink( x-r, y+yy );
+		if ( !link.empty() )
+			return link;
+	}
+	return lString32::empty_str;
+}
+
+lString32 DocViewNative::getLink( int x, int y )
+{
+	ldomXPointer p = _docview->getNodeByPoint( lvPoint(x, y) );
+	if ( p.isNull() )
+		return lString32::empty_str;
+	lString32 href = p.getHRef();
+	return href;
+}
+
+// The maximum allowed value for multiplying the width and height of the image
+#define MAX_IMAGE_SIZE_MUL 16777216		// 4096 x 4096 px, for now 4K is enough
+
+// checks whether point belongs to image: if found, returns true, and _currentImage is set to image
+bool DocViewNative::checkImage(int x, int y, int bufWidth, int bufHeight, int &dx, int &dy, bool & needRotate)
+{
+	_currentImage = _docview->getImageByPoint(lvPoint(x, y));
+	if (_currentImage.isNull())
+		return false;
+	dx = _currentImage->GetWidth();
+	dy = _currentImage->GetHeight();
+	if (dx < 8 && dy < 8) {
+		_currentImage.Clear();
+		return false;
+	}
+	needRotate = false;
+	if (bufWidth <= bufHeight) {
+		// screen == portrait
+		needRotate = 8 * dx > 10 * dy;
+	} else {
+		// screen == landscape
+		needRotate = 10 * dx < 8 * dy;
+	}
+	int scale = 1;
+	if (dx * dy > MAX_IMAGE_SIZE_MUL) {
+		scale = dx * dy / MAX_IMAGE_SIZE_MUL;
+		dx /= scale;
+		dy /= scale;
+	}
+	LVColorDrawBuf * buf = new LVColorDrawBuf(dx, dy);
+	buf->Clear(0xFF000000); // transparent
+	buf->Draw(_currentImage, 0, 0, dx, dy, false);
+	if (needRotate) {
+		int c = dx;
+		dx = dy;
+		dy = c;
+		buf->Rotate(CR_ROTATE_ANGLE_90);
+	}
+	_currentImage = LVCreateDrawBufImageSource(buf, true);
+	return true;
+}
+
+// draws current image to buffer (scaled, panned)
+bool DocViewNative::drawImage(LVDrawBuf * buf, int x, int y, int dx, int dy)
+{
+	if (_currentImage.isNull())
+		return false;
+	return _docview->drawImage(buf, _currentImage, x, y, dx, dy);
+}
+
+// draws icon to buffer
+bool DocViewNative::drawIcon(LVDrawBuf * buf, lvRect & rc, int type) {
+	rc.shrink(rc.width() / 7);
+	lUInt32 light = 0x60C0C0C0;
+	lUInt32 dark = 0x80606060;
+	lUInt32 colors[2];
+	colors[0] = dark;
+	colors[1] = light;
+	// x0  x1  x2  x3
+	int x0 = rc.left;
+	int x1 = rc.left + rc.width() * 4 / 10;
+	int x2 = rc.right -  + rc.width() * 4 / 10;
+	int x3 = rc.right;
+	int y0 = rc.top;
+	int y1 = rc.top + rc.width() * 4 / 10;
+	int y2 = rc.bottom -  + rc.height() * 4 / 10;
+	int y3 = rc.bottom;
+	for (int w = 1; w>=0; w--) {
+		if (type == 1) {
+			// horizontal minus
+			buf->FillRect(x0-w, y1-w, x3+w+1, y1+w+1, colors[w]);
+			buf->FillRect(x0-w, y2-w, x3+w+1, y2+w+1, colors[w]);
+			buf->FillRect(x0-w, y1-w, x0+w+1, y2+w+1, colors[w]);
+			buf->FillRect(x3-w, y1-w, x3+w+1, y2+w+1, colors[w]);
+		} else if (type == 2) {
+			// vertical minus
+			buf->FillRect(x1-w, y0-w, x1+w+1, y3+w+1, colors[w]);
+			buf->FillRect(x2-w, y0-w, x2+w+1, y3+w+1, colors[w]);
+			buf->FillRect(x1-w, y0-w, x2+w+1, y0+w+1, colors[w]);
+			buf->FillRect(x1-w, y3-w, x2+w+1, y3+w+1, colors[w]);
+		} else {
+			// plus
+			buf->FillRect(x0-w, y1-w, x1+w+1, y1+w+1, colors[w]);
+			buf->FillRect(x1-w, y0-w, x1+w+1, y1+w+1, colors[w]);
+			buf->FillRect(x0-w, y1-w, x0+w+1, y2+w+1, colors[w]);
+			buf->FillRect(x1-w, y0-w, x2+w+1, y0+w+1, colors[w]);
+			buf->FillRect(x2-w, y0-w, x2+w+1, y1+w+1, colors[w]);
+			buf->FillRect(x2-w, y1-w, x3+w+1, y1+w+1, colors[w]);
+			buf->FillRect(x3-w, y1-w, x3+w+1, y2+w+1, colors[w]);
+			buf->FillRect(x2-w, y2-w, x3+w+1, y2+w+1, colors[w]);
+			buf->FillRect(x2-w, y2-w, x2+w+1, y3+w+1, colors[w]);
+			buf->FillRect(x1-w, y3-w, x2+w+1, y3+w+1, colors[w]);
+			buf->FillRect(x1-w, y2-w, x1+w+1, y3+w+1, colors[w]);
+			buf->FillRect(x0-w, y2-w, x1+w+1, y2+w+1, colors[w]);
+		}
+	}
+	return true;
+}
+
+// sets current image to null
+bool DocViewNative::closeImage() {
+	if (_currentImage.isNull())
+		return false;
+	_currentImage.Clear();
+	return true;
+}
+
+void DocViewNative::updateBatteryIcons() {
+	CRPropRef props = _docview->propsGetCurrent();
+	lUInt32 textColor = props->getColorDef(PROP_FONT_COLOR, 0x000000);
+	lUInt32 statusColor = props->getColorDef(PROP_STATUS_FONT_COLOR, 0xFF000000);
+	lUInt32 batteryColor = statusColor;
+	if ( batteryColor==0xFF000000 )
+		batteryColor = textColor;
+	int newSize = 28;
+	int w = _docview->GetWidth();
+	int h = _docview->GetHeight();
+	if (w > h)
+		w = h;
+	if (w > 700)
+		newSize = 56;
+	if ( _batteryIconColor != batteryColor || _batteryIconSize != newSize ) { //oldNightMode!=newNightMode
+		_batteryIconColor = batteryColor;
+		_batteryIconSize = newSize;
+		//CRLog::debug("%x->%x, %x->%x: Setting Battery icon color = #%06x", oldTextColor, newTextColor, oldStatusColor, newStatusColor, batteryColor);
+		LVRefVec<LVImageSource> icons;
+		getBatteryIcons(icons, _batteryIconColor, _batteryIconSize);
+		//CRLog::debug("Setting list of Battery icon bitmats");
+		_docview->setBatteryIcons( icons );
+		//CRLog::debug("Setting list of Battery icon bitmats - done");
+	}
+}
+
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /*
  * Class:     org_coolreader_crengine_DocView
@@ -1306,8 +1508,8 @@ JNIEXPORT void JNICALL Java_org_coolreader_crengine_DocView_createDefaultDocumen
 		CRLog::error("Cannot get native view");
 		return;
 	}
-	lString16 title_str = env.fromJavaString(title);
-	lString16 message_str = env.fromJavaString(message);
+	lString32 title_str = env.fromJavaString(title);
+	lString32 message_str = env.fromJavaString(message);
 	p->createDefaultDocument(title_str, message_str);
 }
 
@@ -1326,11 +1528,32 @@ JNIEXPORT jboolean JNICALL Java_org_coolreader_crengine_DocView_loadDocumentInte
     	return JNI_FALSE;
     }
 	DocViewCallback callback( _env, p->_docview, _this );
-	lString16 str = env.fromJavaString(s);
+	lString32 str = env.fromJavaString(s);
     bool res = p->loadDocument(str);
     return res ? JNI_TRUE : JNI_FALSE;
 }
 
+/*
+ * Class:     org_coolreader_crengine_DocView
+ * Method:    loadDocumentFromMemoryInternal
+ * Signature: ([BLjava/lang/String;)Z
+ */
+JNIEXPORT jboolean JNICALL Java_org_coolreader_crengine_DocView_loadDocumentFromMemoryInternal
+		(JNIEnv * _env, jobject _this, jbyteArray buf, jstring contentPath)
+{
+	CRJNIEnv env(_env);
+	DocViewNative * p = getNative(_env, _this);
+	if (!p) {
+		CRLog::error("Cannot get native view");
+		return JNI_FALSE;
+	}
+	DocViewCallback callback( _env, p->_docview, _this );
+	LVStreamRef stream = env.jbyteArrayToStream(buf);
+	lString32 contentPath32 = env.fromJavaString(contentPath);
+	bool res = p->loadDocument(stream, contentPath32);
+	return res ? JNI_TRUE : JNI_FALSE;
+}
+
 /*
  * Class:     org_coolreader_crengine_DocView
  * Method:    getSettings
@@ -1350,6 +1573,25 @@ JNIEXPORT jobject JNICALL Java_org_coolreader_crengine_DocView_getSettingsIntern
     return env.toJavaProperties(props);
 }
 
+/*
+ * Class:     org_coolreader_crengine_DocView
+ * Method:    getDocPropsInternal
+ * Signature: ()Ljava/util/Properties;
+ */
+JNIEXPORT jobject JNICALL Java_org_coolreader_crengine_DocView_getDocPropsInternal
+  (JNIEnv * _env, jobject _this)
+{
+    CRLog::trace("DocView_getDocPropsInternal");
+    CRJNIEnv env(_env);
+    DocViewNative * p = getNative(_env, _this);
+    if (!p) {
+        CRLog::error("Cannot get native view");
+        return NULL;
+    }
+    CRPropRef props = p->_docview->getDocProps();
+    return env.toJavaProperties(props);
+}
+
 #define PROP_NIGHT_MODE "crengine.night.mode"
 
 /*
@@ -1511,9 +1753,9 @@ JNIEXPORT jobject JNICALL Java_org_coolreader_crengine_DocView_getCurrentPageBoo
 	if ( ptr.isNull() )
 		return NULL;
 	CRBookmark bm(ptr);
-	lString16 comment;
-    lString16 titleText;
-    lString16 posText;
+	lString32 comment;
+    lString32 titleText;
+    lString32 posText;
     bm.setType( bmkt_pos );
     if ( p->_docview->getBookmarkPosText( ptr, titleText, posText ) ) {
          bm.setTitleText( titleText );
@@ -1548,10 +1790,10 @@ JNIEXPORT jobject JNICALL Java_org_coolreader_crengine_DocView_getCurrentPageBoo
 /*
  * Class:     org_coolreader_crengine_DocView
  * Method:    updateBookInfoInternal
- * Signature: (Lorg/coolreader/crengine/BookInfo;)V
+ * Signature: (Lorg/coolreader/crengine/BookInfo;Z)V
  */
 JNIEXPORT void JNICALL Java_org_coolreader_crengine_DocView_updateBookInfoInternal
-  (JNIEnv * _env, jobject _this, jobject _info)
+  (JNIEnv * _env, jobject _this, jobject _info, jboolean _updatePath)
 {
 	CRJNIEnv env(_env);
     DocViewNative * p = getNative(_env, _this);
@@ -1579,6 +1821,60 @@ JNIEXPORT void JNICALL Java_org_coolreader_crengine_DocView_updateBookInfoIntern
     CRStringField languageField(fileinfo,"language");
     if (languageField.get().empty())
     	languageField.set(p->_docview->getLanguage());
+    if (doc_format_fb2 == p->_docview->getDocFormat()) {
+        // TODO: may be fb3 too...
+        CRStringField genresField(fileinfo, "genres");
+        lString32 genres = p->_docview->getKeywords();
+        // keywords separated by "\n", see lvtinydom.cpp:
+        //    lString32 extractDocKeywords( ldomDocument * doc )
+        int pos = genres.pos('\n');
+        while (pos > 0) {
+            genres[pos] = '|';
+            pos = genres.pos('\n', pos + 1);
+        }
+        genresField.set(genres);
+    }
+    CRStringField descriptionField(fileinfo,"description");
+    descriptionField.set(p->_docview->getDescription());
+    CRLongField crc32Field(fileinfo,"crc32");
+    crc32Field.set((lInt64)p->_docview->getFileCRC32());
+    jclass clazz_docformat = env->FindClass("org/coolreader/crengine/DocumentFormat");
+    jmethodID methodId_byId = NULL;
+    if (NULL != clazz_docformat) {
+        methodId_byId = env->GetStaticMethodID(clazz_docformat, "byId", "(I)Lorg/coolreader/crengine/DocumentFormat;");
+    }
+    if (NULL != methodId_byId) {
+        jobject jformat = env->CallStaticObjectMethod(clazz_docformat, methodId_byId, p->_docview->getDocFormat());
+        CRObjectField formatField(fileinfo, "format", "Lorg/coolreader/crengine/DocumentFormat;");
+        formatField.set(jformat);
+    }
+    if (_updatePath) {
+		CRPropRef doc_props = p->_docview->getDocProps();
+		if (!doc_props.isNull()) {
+			bool isArchive = !doc_props->getStringDef(DOC_PROP_ARC_NAME, "").empty();
+			if (isArchive) {
+				lString32 arcname = doc_props->getStringDef(DOC_PROP_ARC_NAME, "");
+				int arcsize = doc_props->getIntDef(DOC_PROP_ARC_SIZE, 0);
+				CRStringField arcnameField(fileinfo, "arcname");
+				arcnameField.set(arcname);
+				CRLongField arcsizeField(fileinfo, "arcsize");
+				arcsizeField.set(arcsize);
+			}
+			lString32 pathname = doc_props->getStringDef(DOC_PROP_FILE_NAME, "");
+			lString32 filename = LVExtractFilename(pathname);
+			int size = doc_props->getIntDef(DOC_PROP_FILE_SIZE, 0);
+			CRBooleanField isArchiveField(fileinfo, "isArchive");
+			isArchiveField.set(isArchive);
+			CRStringField pathnameField(fileinfo, "pathname");
+			CRStringField pathField(fileinfo, "path");
+			pathnameField.set(pathname);
+			pathField.set(pathname);
+			CRStringField filenameField(fileinfo, "filename");
+			filenameField.set(filename);
+			CRLongField sizeField(fileinfo, "size");
+			sizeField.set(size);
+		}
+	}
 }
 
 /*
@@ -1598,7 +1894,7 @@ JNIEXPORT jboolean JNICALL Java_org_coolreader_crengine_DocView_goToPositionInte
 	if ( !p->_docview->isDocumentOpened() )
 		return JNI_FALSE;
 	DocViewCallback callback( _env, p->_docview, _this );
-	lString16 str = env.fromJavaString(jstr);
+	lString32 str = env.fromJavaString(jstr);
     ldomXPointer bm = p->_docview->getDocument()->createXPointer(str);
 	if ( bm.isNull() )
 		return JNI_FALSE;
@@ -1614,7 +1910,7 @@ JNIEXPORT jboolean JNICALL Java_org_coolreader_crengine_DocView_goToPositionInte
  * Signature: (Ljava/lang/String;)Lorg/coolreader/crengine/DocView/PositionProperties;
  */
 JNIEXPORT jobject JNICALL Java_org_coolreader_crengine_DocView_getPositionPropsInternal
-    (JNIEnv * _env, jobject _this, jstring _path)
+    (JNIEnv * _env, jobject _this, jstring _path, jboolean precise)
 {
     CRJNIEnv env(_env);
     DocViewNative * p = getNative(_env, _this);
@@ -1632,7 +1928,7 @@ JNIEXPORT jobject JNICALL Java_org_coolreader_crengine_DocView_getPositionPropsI
 		return obj;
 	}
 	DocViewCallback callback( _env, p->_docview, _this );
-    lString16 str = env.fromJavaString(_path);
+    lString32 str = env.fromJavaString(_path);
     ldomXPointer bm;
     bool useCurPos = false; // use current Y position for scroll view mode
     p->_docview->checkPos();
@@ -1641,7 +1937,9 @@ JNIEXPORT jobject JNICALL Java_org_coolreader_crengine_DocView_getPositionPropsI
     } else {
         useCurPos = p->_docview->getViewMode()==DVM_SCROLL;
         if ( !useCurPos ) {
-            bm = p->_docview->getBookmark();
+            // For page turn command 'precise' must be 'false' for maximum speed.
+            // For bookmarking 'precise' must be 'true' for more accurate result (y position).
+            bm = p->_docview->getBookmark(precise);
             if ( bm.isNull() ) {
                 CRLog::error("getPositionPropsInternal: Cannot get current position bookmark");
             }
@@ -1657,10 +1955,28 @@ JNIEXPORT jobject JNICALL Java_org_coolreader_crengine_DocView_getPositionPropsI
     CRIntField(v,"pageNumber").set(p->_docview->getCurPage());
     CRIntField(v,"pageCount").set(p->_docview->getPageCount());
     CRIntField(v,"pageMode").set(p->_docview->getViewMode()==DVM_PAGES ? p->_docview->getVisiblePageCount() : 0);
-    CRIntField(v,"charCount").set(p->_docview->getCurrentPageCharCount());
-    CRIntField(v,"imageCount").set(p->_docview->getCurrentPageImageCount());
-    CRStringField(v,"pageText").set(p->_docview->getPageText(false, -1));
-	return obj;
+    if (precise) {
+        // In some cases, function LVDocView::getPageDocumentRange() is rather slow,
+        // so if we don't need these fields 'charCount', 'imageCount' and 'pageText'
+        // in the returned result, we just don't call it.
+        // In this case, 'precise' must be 'false'.
+        p->_docview->getMutex().lock();
+        LVRef<ldomXRange> range = p->_docview->getPageDocumentRange(-1);
+        p->_docview->getMutex().unlock();
+        lString32 text;
+        if (!range.isNull())
+            text = range->getRangeText();
+        int charCount = 0;
+        for (int i = 0; i < text.length(); i++) {
+            lChar32 ch = text[i];
+            if (ch >= '0')
+                charCount++;
+        }
+        CRIntField(v, "charCount").set(charCount);
+        CRIntField(v, "imageCount").set(p->_docview->getPageImageCount(range));
+        CRStringField(v, "pageText").set(text);
+    }
+    return obj;
 }
 
 /*
@@ -1725,10 +2041,10 @@ JNIEXPORT jboolean JNICALL Java_org_coolreader_crengine_DocView_findTextInternal
 /*
  * Class:     org_coolreader_crengine_DocView
  * Method:    setBatteryStateInternal
- * Signature: (I)V
+ * Signature: (III)V
  */
 JNIEXPORT void JNICALL Java_org_coolreader_crengine_DocView_setBatteryStateInternal
-  (JNIEnv * _env, jobject _this, jint state)
+  (JNIEnv * _env, jobject _this, jint state, jint chargingConn, jint chargeLevel)
 {
     CRJNIEnv env(_env);
     DocViewNative * p = getNative(_env, _this);
@@ -1736,7 +2052,7 @@ JNIEXPORT void JNICALL Java_org_coolreader_crengine_DocView_setBatteryStateInter
     	CRLog::error("Cannot get native view");
     	return;
     }
-    p->_docview->setBatteryState(state);
+    p->_docview->setBatteryState(state, chargingConn, chargeLevel);
 }
 
 /*
@@ -1843,24 +2159,24 @@ JNIEXPORT void JNICALL Java_org_coolreader_crengine_DocView_updateSelectionInter
         r.sort();
 		if ( !r.getStart().isVisibleWordStart() )
 			r.getStart().prevVisibleWordStart();
-		//lString16 start = r.getStart().toString();
+		//lString32 start = r.getStart().toString();
 		if ( !r.getEnd().isVisibleWordEnd() )
 			r.getEnd().nextVisibleWordEnd();
         if ( r.isNull() )
             return;
-        //lString16 end = r.getEnd().toString();
+        //lString32 end = r.getEnd().toString();
         //CRLog::debug("Range: %s - %s", UnicodeToUtf8(start).c_str(), UnicodeToUtf8(end).c_str());
         r.setFlags(1);
         p->_docview->selectRange( r );
         int page = p->_docview->getBookmarkPage(startp);
         int pages = p->_docview->getPageCount();
-        lString16 titleText;
-        lString16 posText;
+        lString32 titleText;
+        lString32 posText;
         p->_docview->getBookmarkPosText(startp, titleText, posText);
         int percent = 0;
         if ( pages>1 )
         	percent = 10000 * page/(pages-1);
-        lString16 selText = r.getRangeText( '\n', 8192 );
+        lString32 selText = r.getRangeText( '\n', 8192 );
         sel_percent.set(percent);
     	sel_startPos.set( r.getStart().toString() );
     	sel_endPos.set( r.getEnd().toString() );
@@ -1916,13 +2232,13 @@ JNIEXPORT jboolean JNICALL Java_org_coolreader_crengine_DocView_moveSelectionInt
 
                 int page = p->_docview->getBookmarkPage(currSel.getStart());
                 int pages = p->_docview->getPageCount();
-                lString16 titleText;
-                lString16 posText;
+                lString32 titleText;
+                lString32 posText;
                 p->_docview->getBookmarkPosText(currSel.getStart(), titleText, posText);
                 int percent = 0;
                 if ( pages>1 )
                 	percent = 10000 * page/(pages-1);
-                lString16 selText = currSel.getRangeText( '\n', 8192 );
+                lString32 selText = currSel.getRangeText( '\n', 8192 );
                 sel_percent.set(percent);
             	sel_text.set(selText);
             	sel_chapter.set(titleText);
@@ -1934,149 +2250,6 @@ JNIEXPORT jboolean JNICALL Java_org_coolreader_crengine_DocView_moveSelectionInt
     return JNI_FALSE;
 }
 
-
-lString16 DocViewNative::getLink( int x, int y, int r )
-{
-	int step = 5;
-	int n = r / step;
-	r = n * step;
-	if ( r==0 )
-		return getLink(x, y);
-	lString16 link;
-	for ( int xx = -r; xx<=r; xx+=step ) {
-		link = getLink( x+xx, y-r );
-		if ( !link.empty() )
-			return link;
-		link = getLink( x+xx, y+r );
-		if ( !link.empty() )
-			return link;
-	}
-	for ( int yy = -r + step; yy<=r - step; yy+=step ) {
-		link = getLink( x+r, y+yy );
-		if ( !link.empty() )
-			return link;
-		link = getLink( x-r, y+yy );
-		if ( !link.empty() )
-			return link;
-	}
-	return lString16::empty_str;
-}
-
-lString16 DocViewNative::getLink( int x, int y )
-{
-	ldomXPointer p = _docview->getNodeByPoint( lvPoint(x, y) );
-	if ( p.isNull() )
-		return lString16::empty_str;
-	lString16 href = p.getHRef();
-	return href;
-}
-
-#define MAX_IMAGE_BUF_SIZE 1200000
-
-// checks whether point belongs to image: if found, returns true, and _currentImage is set to image
-bool DocViewNative::checkImage(int x, int y, int bufWidth, int bufHeight, int &dx, int &dy, bool & needRotate)
-{
-	_currentImage = _docview->getImageByPoint(lvPoint(x, y));
-	if (_currentImage.isNull())
-		return false;
-	dx = _currentImage->GetWidth();
-	dy = _currentImage->GetHeight();
-	if (dx < 8 && dy < 8) {
-		_currentImage.Clear();
-		return false;
-	}
-	needRotate = false;
-	if (bufWidth <= bufHeight) {
-		// screen == portrait
-		needRotate = 8 * dx > 10 * dy;
-	} else {
-		// screen == landscape
-		needRotate = 10 * dx < 8 * dy;
-	}
-	int scale = 1;
-	if (dx * dy > MAX_IMAGE_BUF_SIZE) {
-		scale = dx * dy /  MAX_IMAGE_BUF_SIZE;
-		dx /= scale;
-		dy /= scale;
-	}
-	LVColorDrawBuf * buf = new LVColorDrawBuf(dx, dy);
-	buf->Clear(0xFF000000); // transparent
-	buf->Draw(_currentImage, 0, 0, dx, dy, false);
-	if (needRotate) {
-		int c = dx;
-		dx = dy;
-		dy = c;
-		buf->Rotate(CR_ROTATE_ANGLE_90);
-	}
-	_currentImage = LVCreateDrawBufImageSource(buf, true);
-	return true;
-}
-
-// draws current image to buffer (scaled, panned)
-bool DocViewNative::drawImage(LVDrawBuf * buf, int x, int y, int dx, int dy)
-{
-	if (_currentImage.isNull())
-		return false;
-	return _docview->drawImage(buf, _currentImage, x, y, dx, dy);
-}
-
-// draws icon to buffer
-bool DocViewNative::drawIcon(LVDrawBuf * buf, lvRect & rc, int type) {
-	rc.shrink(rc.width() / 7);
-	lUInt32 light = 0x60C0C0C0;
-	lUInt32 dark = 0x80606060;
-	lUInt32 colors[2];
-	colors[0] = dark;
-	colors[1] = light;
-	// x0  x1  x2  x3
-	int x0 = rc.left;
-	int x1 = rc.left + rc.width() * 4 / 10;
-	int x2 = rc.right -  + rc.width() * 4 / 10;
-	int x3 = rc.right;
-	int y0 = rc.top;
-	int y1 = rc.top + rc.width() * 4 / 10;
-	int y2 = rc.bottom -  + rc.height() * 4 / 10;
-	int y3 = rc.bottom;
-	for (int w = 1; w>=0; w--) {
-		if (type == 1) {
-			// horizontal minus
-			buf->FillRect(x0-w, y1-w, x3+w+1, y1+w+1, colors[w]);
-			buf->FillRect(x0-w, y2-w, x3+w+1, y2+w+1, colors[w]);
-			buf->FillRect(x0-w, y1-w, x0+w+1, y2+w+1, colors[w]);
-			buf->FillRect(x3-w, y1-w, x3+w+1, y2+w+1, colors[w]);
-		} else if (type == 2) {
-			// vertical minus
-			buf->FillRect(x1-w, y0-w, x1+w+1, y3+w+1, colors[w]);
-			buf->FillRect(x2-w, y0-w, x2+w+1, y3+w+1, colors[w]);
-			buf->FillRect(x1-w, y0-w, x2+w+1, y0+w+1, colors[w]);
-			buf->FillRect(x1-w, y3-w, x2+w+1, y3+w+1, colors[w]);
-		} else {
-			// plus
-			buf->FillRect(x0-w, y1-w, x1+w+1, y1+w+1, colors[w]);
-			buf->FillRect(x1-w, y0-w, x1+w+1, y1+w+1, colors[w]);
-			buf->FillRect(x0-w, y1-w, x0+w+1, y2+w+1, colors[w]);
-			buf->FillRect(x1-w, y0-w, x2+w+1, y0+w+1, colors[w]);
-			buf->FillRect(x2-w, y0-w, x2+w+1, y1+w+1, colors[w]);
-			buf->FillRect(x2-w, y1-w, x3+w+1, y1+w+1, colors[w]);
-			buf->FillRect(x3-w, y1-w, x3+w+1, y2+w+1, colors[w]);
-			buf->FillRect(x2-w, y2-w, x3+w+1, y2+w+1, colors[w]);
-			buf->FillRect(x2-w, y2-w, x2+w+1, y3+w+1, colors[w]);
-			buf->FillRect(x1-w, y3-w, x2+w+1, y3+w+1, colors[w]);
-			buf->FillRect(x1-w, y2-w, x1+w+1, y3+w+1, colors[w]);
-			buf->FillRect(x0-w, y2-w, x1+w+1, y2+w+1, colors[w]);
-		}
-	}
-	return true;
-}
-
-// sets current image to null
-bool DocViewNative::closeImage() {
-	if (_currentImage.isNull())
-		return false;
-	_currentImage.Clear();
-	return true;
-}
-
 /*
  * Class:     org_coolreader_crengine_DocView
  * Method:    checkLinkInternal
@@ -2091,7 +2264,7 @@ JNIEXPORT jstring JNICALL Java_org_coolreader_crengine_DocView_checkLinkInternal
     	CRLog::error("Cannot get native view");
     	return NULL;
     }
-    lString16 link;
+    lString32 link;
     for ( int r=0; r<=delta; r+=5 ) {
     	link = p->getLink(x, y, r);
     	if ( !link.empty() )
@@ -2114,7 +2287,7 @@ JNIEXPORT jint JNICALL Java_org_coolreader_crengine_DocView_goLinkInternal
     	CRLog::error("Cannot get native view");
     	return 0;
     }
-    lString16 link = env.fromJavaString(_link);
+    lString32 link = env.fromJavaString(_link);
     bool res = p->_docview->goLink( link, true );
     return res ? 1 : 0;
 }
@@ -2150,30 +2323,22 @@ JNIEXPORT void JNICALL Java_org_coolreader_crengine_DocView_hilightBookmarksInte
     p->_docview->setBookmarkList(bookmarks);
 }
 
-
-
-void DocViewNative::updateBatteryIcons() {
-	CRPropRef props = _docview->propsGetCurrent();
-	lUInt32 textColor = props->getColorDef(PROP_FONT_COLOR, 0x000000);
-	lUInt32 statusColor = props->getColorDef(PROP_STATUS_FONT_COLOR, 0xFF000000);
-	lUInt32 batteryColor = statusColor;
-	if ( batteryColor==0xFF000000 )
-		batteryColor = textColor;
-	int newSize = 28;
-	int w = _docview->GetWidth();
-	int h = _docview->GetHeight();
-	if (w > h)
-		w = h;
-	if (w > 700)
-		newSize = 56;
-	if ( _batteryIconColor != batteryColor || _batteryIconSize != newSize ) { //oldNightMode!=newNightMode
-		_batteryIconColor = batteryColor;
-		_batteryIconSize = newSize;
-		//CRLog::debug("%x->%x, %x->%x: Setting Battery icon color = #%06x", oldTextColor, newTextColor, oldStatusColor, newStatusColor, batteryColor);
-	    LVRefVec<LVImageSource> icons;
-	    getBatteryIcons(icons, _batteryIconColor, _batteryIconSize);
-		//CRLog::debug("Setting list of Battery icon bitmats");
-	   _docview->setBatteryIcons( icons );
-		//CRLog::debug("Setting list of Battery icon bitmats - done");
-	}
+/*
+ * Class:     org_coolreader_crengine_DocView
+ * Method:    isTimeChangedInternal
+ * Signature: ()Z
+ */
+JNIEXPORT jboolean JNICALL Java_org_coolreader_crengine_DocView_isTimeChangedInternal
+  (JNIEnv * _env, jobject _this) {
+    ;
+    DocViewNative * p = getNative(_env, _this);
+    if (!p) {
+        CRLog::error("Cannot get native view");
+        return JNI_FALSE;
+    }
+    return p->_docview->isTimeChanged() ? JNI_TRUE : JNI_FALSE;
 }
+
+#ifdef __cplusplus
+}
+#endif
