@@ -1,20 +1,35 @@
 package org.coolreader.crengine;
 
 import android.content.Context;
-import android.content.DialogInterface;
-import android.content.DialogInterface.OnCancelListener;
-import android.view.*;
+import android.view.ContextMenu;
+import android.view.GestureDetector;
 import android.view.GestureDetector.SimpleOnGestureListener;
-import android.view.MenuItem.OnMenuItemClickListener;
-import android.widget.*;
+import android.view.KeyEvent;
+import android.view.LayoutInflater;
+import android.view.MenuInflater;
+import android.view.MenuItem;
+import android.view.MotionEvent;
+import android.view.View;
+import android.view.ViewGroup;
+import android.widget.Adapter;
+import android.widget.ImageView;
+import android.widget.LinearLayout;
+import android.widget.ListView;
+import android.widget.TextView;
+
 import org.coolreader.CoolReader;
 import org.coolreader.R;
-import org.coolreader.crengine.CoverpageManager.CoverpageReadyListener;
 import org.coolreader.crengine.OPDSUtil.DocInfo;
 import org.coolreader.crengine.OPDSUtil.DownloadCallback;
 import org.coolreader.crengine.OPDSUtil.EntryInfo;
 import org.coolreader.db.CRDBService;
-import org.coolreader.plugins.*;
+import org.coolreader.plugins.AuthenticationCallback;
+import org.coolreader.plugins.BookInfoCallback;
+import org.coolreader.plugins.FileInfoCallback;
+import org.coolreader.plugins.OnlineStoreBook;
+import org.coolreader.plugins.OnlineStoreBookInfo;
+import org.coolreader.plugins.OnlineStorePluginManager;
+import org.coolreader.plugins.OnlineStoreWrapper;
 import org.koekak.android.ebookdownloader.SonyBookSelector;
 
 import java.io.File;
@@ -29,10 +44,12 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 	
 	Engine mEngine;
 	Scanner mScanner;
+	Scanner.ScanControl mScanControl;
 	CoolReader mActivity;
 	LayoutInflater mInflater;
 	History mHistory;
 	ListView mListView;
+	boolean mHideEmptyGenres;
 
 	public static final int MAX_SUBDIR_LEN = 32;
 	
@@ -47,29 +64,24 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 	        setFocusableInTouchMode(true);
 	        requestFocus();
 	        
-	        setOnItemLongClickListener(new OnItemLongClickListener() {
+	        setOnItemLongClickListener((arg0, arg1, position, id) -> {
+				log.d("onItemLongClick("+position+")");
+				//return super.performItemClick(view, position, id);
+				FileInfo item = (FileInfo) getAdapter().getItem(position);
+				if ( item==null )
+					return false;
+				//openContextMenu(_this);
+				//mActivity.loadDocument(item);
+				selectedItem = item;
 
-				@Override
-				public boolean onItemLongClick(AdapterView<?> arg0, View arg1,
-						int position, long id) {
-					log.d("onItemLongClick("+position+")");
-					//return super.performItemClick(view, position, id);
-					FileInfo item = (FileInfo) getAdapter().getItem(position);
-					if ( item==null )
-						return false;
-					//openContextMenu(_this);
-					//mActivity.loadDocument(item);
-					selectedItem = item;
-					
-					boolean bookInfoDialogEnabled = true; // TODO: it's for debug
-					if (!item.isDirectory && !item.isOPDSBook() && bookInfoDialogEnabled && !item.isOnlineCatalogPluginDir()) {
-						mActivity.editBookInfo(currDirectory, item);
-						return true;
-					}
-					
-					showContextMenu();
+				boolean bookInfoDialogEnabled = true; // TODO: it's for debug
+				if (!item.isDirectory && !item.isOPDSBook() && bookInfoDialogEnabled && !item.isOnlineCatalogPluginDir()) {
+					mActivity.editBookInfo(currDirectory, item);
 					return true;
 				}
+
+				showContextMenu();
+				return true;
 			});
 			setChoiceMode(CHOICE_MODE_SINGLE);
 		}
@@ -101,12 +113,16 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 			    inflater.inflate(R.menu.cr3_file_browser_context_menu, menu);
 			    menu.setHeaderTitle(mActivity.getString(R.string.context_menu_title_book));
 		    }
-		    for ( int i=0; i<menu.size(); i++ ) {
-		    	menu.getItem(i).setOnMenuItemClickListener(new OnMenuItemClickListener() {
-					public boolean onMenuItemClick(MenuItem item) {
-						onContextItemSelected(item);
-						return true;
+			for ( int i=0; i<menu.size(); i++ ) {
+				MenuItem menuItem = menu.getItem(i);
+				if (selectedItem!=null && selectedItem.isDirectory && selectedItem.isSpecialDir()) {
+					if (menuItem.getItemId() == R.id.folder_delete || menuItem.getItemId() == R.id.folder_to_favorites) {
+						menuItem.setVisible(false);
 					}
+				}
+				menu.getItem(i).setOnMenuItemClickListener(item -> {
+					onContextItemSelected(item);
+					return true;
 				});
 		    }
 		}
@@ -128,7 +144,7 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 			else if (item.isOnlineCatalogPluginBook())
 				showOnlineCatalogBookDialog(item);
 			else
-				mActivity.loadDocument(item);
+				mActivity.loadDocument(item, true);
 			return true;
 		}
 
@@ -166,28 +182,27 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 	}
 	
 	CoverpageManager.CoverpageReadyListener coverpageListener;
-	public FileBrowser(CoolReader activity, Engine engine, Scanner scanner, History history) {
+	public FileBrowser(CoolReader activity, Engine engine, Scanner scanner, History history, boolean hideEmptyGenres) {
 		super(activity);
 		this.mActivity = activity;
 		this.mEngine = engine;
 		this.mScanner = scanner;
+		this.mScanControl = new Scanner.ScanControl();
 		this.mInflater = LayoutInflater.from(activity);// activity.getLayoutInflater();
 		this.mHistory = history;
 		this.mCoverpageManager = Services.getCoverpageManager();
+		this.mHideEmptyGenres = hideEmptyGenres;
 
-		coverpageListener =	new CoverpageReadyListener() {
-			@Override
-			public void onCoverpagesReady(ArrayList<CoverpageManager.ImageItem> files) {
-				if (currDirectory == null)
-					return;
-				boolean found = false;
-				for (CoverpageManager.ImageItem file : files) {
-					if (currDirectory.findItemByPathName(file.file.getPathName()) != null)
-						found = true;
-				}
-				if (found) // && mListView.getS
-					invalidateAdapter(currentListAdapter);
+		coverpageListener = files -> {
+			if (currDirectory == null)
+				return;
+			boolean found = false;
+			for (CoverpageManager.ImageItem file : files) {
+				if (currDirectory.findItemByPathName(file.file.getPathName()) != null)
+					found = true;
 			}
+			if (found) // && mListView.getS
+				invalidateAdapter(currentListAdapter);
 		};
 		this.mCoverpageManager.addCoverpageReadyListener(coverpageListener);
 		super.onAttachedToWindow();
@@ -199,13 +214,25 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 		showDirectory( null, null );
 
 	}
-	
+
+	public void setHideEmptyGenres(boolean value) {
+		mHideEmptyGenres = value;
+		if (null != currDirectory && (currDirectory.isBooksByGenreDir() || currDirectory.isBooksByGenreRoot())) {
+			// update
+			showDirectory(currDirectory, null);
+		}
+	}
+
 	public void onClose() {
+		mScanControl.stop();
 		this.mCoverpageManager.removeCoverpageReadyListener(coverpageListener);
 		coverpageListener = null;
 		super.onDetachedFromWindow();
 	}
 
+	public void stopCurrentScan() {
+		mScanControl.stop();
+	}
 
 	public CoverpageManager getCoverpageManager() {
 		return mCoverpageManager;
@@ -218,15 +245,12 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 			progress.hide();
 		mListView = new FileBrowserListView(mActivity);
 		final GestureDetector detector = new GestureDetector(new MyGestureListener());
-		mListView.setOnTouchListener(new ListView.OnTouchListener() {
-			@Override
-			public boolean onTouch(View v, MotionEvent event) {
-				try {
-					return detector.onTouchEvent(event);
-				} catch (Exception e) {
-					L.e("Exception in onTouch", e);
-					return false;
-				}
+		mListView.setOnTouchListener((v, event) -> {
+			try {
+				return detector.onTouchEvent(event);
+			} catch (Exception e) {
+				L.e("Exception in onTouch", e);
+				return false;
 			}
 		});
 		if (currentListAdapter == null || recreateAdapter) {
@@ -262,7 +286,7 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 			if ( selectedItem.isOPDSDir() )
 				showOPDSDir(selectedItem, null);
 			else
-				mActivity.loadDocument(selectedItem);
+				mActivity.loadDocument(selectedItem, true);
 			return true;
 		case R.id.book_sort_order:
 			mActivity.showToast("Sorry, sort order selection is not yet implemented");
@@ -318,6 +342,9 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
             log.d("folder_to_favorites menu item selected");
             addToFavorites(selectedItem);
             return true;
+		case R.id.folder_delete:
+			mActivity.askDeleteFolder(selectedItem);
+			return true;
 		}
 		return false;
 	}
@@ -329,15 +356,12 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
     public void refreshOPDSRootDirectory(final boolean showInBrowser) {
 		final FileInfo opdsRoot = mScanner.getOPDSRoot();
 		if (opdsRoot != null) {
-			mActivity.getDB().loadOPDSCatalogs(new CRDBService.OPDSCatalogsLoadingCallback() {
-				@Override
-				public void onOPDSCatalogsLoaded(ArrayList<FileInfo> catalogs) {
-					opdsRoot.clear();
-					for (FileInfo f : catalogs)
-						opdsRoot.addDir(f);
-					if (showInBrowser || (currDirectory!=null && currDirectory.isOPDSRoot()))
-						showDirectory(opdsRoot, null);
-				}
+			mActivity.getDB().loadOPDSCatalogs(catalogs -> {
+				opdsRoot.clear();
+				for (FileInfo f : catalogs)
+					opdsRoot.addDir(f);
+				if (showInBrowser || (currDirectory!=null && currDirectory.isOPDSRoot()))
+					showDirectory(opdsRoot, null);
 			});
 		}
 	}
@@ -354,9 +378,10 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 
 	public void showParentDirectory()
 	{
-		if (currDirectory == null || currDirectory.parent == null || currDirectory.parent.isRootDir())
+		if (currDirectory == null || currDirectory.parent == null || currDirectory.parent.isRootDir()) {
+			mScanControl.stop();
 			mActivity.showRootWindow();
-		else
+		} else
 			showDirectory(currDirectory.parent, currDirectory);
 	}
 	
@@ -410,19 +435,16 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 	
 	public void showFindBookDialog()
 	{
-		BookSearchDialog dlg = new BookSearchDialog( mActivity, new BookSearchDialog.SearchCallback() {
-			@Override
-			public void done(FileInfo[] results) {
-				if (results != null) {
-					if (results.length == 0) {
-						mActivity.showToast(R.string.dlg_book_search_not_found);
-					} else {
-						showSearchResult(results);
-					}
+		BookSearchDialog dlg = new BookSearchDialog( mActivity, results -> {
+			if (results != null) {
+				if (results.length == 0) {
+					mActivity.showToast(R.string.dlg_book_search_not_found);
 				} else {
-					if (currDirectory == null || currDirectory.isRootDir())
-						mActivity.showRootWindow();
+					showSearchResult(results);
 				}
+			} else {
+				if (currDirectory == null || currDirectory.isRootDir())
+					mActivity.showRootWindow();
 			}
 		});
 		dlg.show();
@@ -455,12 +477,7 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 	}
 	
 	private void openPluginDirectoryWithLoginDialog(final OnlineStoreWrapper plugin, final FileInfo dir) {
-		OnlineStoreLoginDialog dlg = new OnlineStoreLoginDialog(mActivity, plugin, new Runnable() {
-			@Override
-			public void run() {
-				openPluginDirectory(plugin, dir);
-			}
-		});
+		OnlineStoreLoginDialog dlg = new OnlineStoreLoginDialog(mActivity, plugin, () -> openPluginDirectory(plugin, dir));
 		dlg.show();
 	}
 
@@ -509,12 +526,9 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 		log.v("showOPDSRootDirectory()");
 		final FileInfo opdsRoot = mScanner.getOPDSRoot();
 		if (opdsRoot != null) {
-			mActivity.getDB().loadOPDSCatalogs(new CRDBService.OPDSCatalogsLoadingCallback() {
-				@Override
-				public void onOPDSCatalogsLoaded(ArrayList<FileInfo> catalogs) {
-					opdsRoot.setItems(catalogs);
-					showDirectoryInternal(opdsRoot, null);
-				}
+			mActivity.getDB().loadOPDSCatalogs(catalogs -> {
+				opdsRoot.setItems(catalogs);
+				showDirectoryInternal(opdsRoot, null);
 			});
 		}
 	}
@@ -565,13 +579,11 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 			}, 
 			mActivity.getString(R.string.mi_book_sort_order), 
 			ReaderView.PROP_APP_BOOK_SORT_ORDER).add(optionValues, optionLabels); 
-		dlg.setOnChangeHandler(new Runnable() {
-			public void run() {
-				final String newValue = properties.getProperty(ReaderView.PROP_APP_BOOK_SORT_ORDER);
-				if ( newValue!=null && oldValue!=null && !newValue.equals(oldValue) ) {
-					log.d("New sort order: " + newValue);
-					setSortOrder(newValue);
-				}
+		dlg.setOnChangeHandler(() -> {
+			final String newValue = properties.getProperty(ReaderView.PROP_APP_BOOK_SORT_ORDER);
+			if ( newValue!=null && oldValue!=null && !newValue.equals(oldValue) ) {
+				log.d("New sort order: " + newValue);
+				setSortOrder(newValue);
 			}
 		});
 		dlg.onSelect();
@@ -581,12 +593,7 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 		
 		if ( fileOrDir.fileCount()>0 || fileOrDir.dirCount()>0 ) {
 			// already downloaded
-			BackgroundThread.instance().executeGUI(new Runnable() {
-				@Override
-				public void run() {
-					showDirectoryInternal(fileOrDir, itemToSelect);					
-				}
-			});
+			BackgroundThread.instance().executeGUI(() -> showDirectoryInternal(fileOrDir, itemToSelect));
 			return;
 		}
 		
@@ -612,7 +619,7 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 							log.w("current directory has been changed: ignore downloaded items");
 							return false;
 						}
-						ArrayList<FileInfo> items = new ArrayList<FileInfo>();
+						ArrayList<FileInfo> items = new ArrayList<>();
 						for ( EntryInfo entry : entries ) {
 							OPDSUtil.LinkInfo acquisition = entry.getBestAcquisitionLink();
 							if ( acquisition!=null ) {
@@ -713,9 +720,9 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 						mScanner.listDirectory(dir);
 						FileInfo item = dir.findItemByPathName(file.getAbsolutePath());
 						if ( item!=null )
-							mActivity.loadDocument(item);
+							mActivity.loadDocument(item, true);
 						else
-							mActivity.loadDocument(fi);
+							mActivity.loadDocument(fi, true);
 					}
 
 					@Override
@@ -741,34 +748,45 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 	
 	private class ItemGroupsLoadingCallback implements CRDBService.ItemGroupsLoadingCallback {
 		private final FileInfo baseDir;
-		public ItemGroupsLoadingCallback(FileInfo baseDir) {
+		private final FileInfo itemToSelect;
+		public ItemGroupsLoadingCallback(FileInfo baseDir, FileInfo itemToSelect) {
 			this.baseDir = baseDir;
+			this.itemToSelect = itemToSelect;
 		}
 		@Override
 		public void onItemGroupsLoaded(FileInfo parent) {
 			baseDir.setItems(parent);
-			showDirectoryInternal(baseDir, null);
+			showDirectoryInternal(baseDir, itemToSelect);
 		}
 	};
 	
 	private class FileInfoLoadingCallback implements CRDBService.FileInfoLoadingCallback {
 		private final FileInfo baseDir;
-		public FileInfoLoadingCallback(FileInfo baseDir) {
+		private final FileInfo itemToSelect;
+		public FileInfoLoadingCallback(FileInfo baseDir, FileInfo itemToSelect) {
 			this.baseDir = baseDir;
+			this.itemToSelect = itemToSelect;
 		}
 		@Override
 		public void onFileInfoListLoaded(ArrayList<FileInfo> list) {
 			baseDir.setItems(list);
-			showDirectoryInternal(baseDir, null);
+			showDirectoryInternal(baseDir, itemToSelect);
 		}
 	};
-	
+
 	@Override
 	public void onChange(FileInfo object, boolean filePropsOnly) {
 		if (currDirectory == null)
 			return;
 		if (!currDirectory.pathNameEquals(object) && !currDirectory.hasItem(object))
 			return;
+		if (currDirectory != object) {
+			if (currDirectory.pathNameEquals(object)) {
+				currDirectory.setItems(object);
+			} else if (currDirectory.hasItem(object)) {
+				currDirectory.updateItem(object);
+			}
+		}
 		// refresh
 		if (filePropsOnly)
 			currentListAdapter.notifyInvalidated();
@@ -824,52 +842,63 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 				showFindBookDialog();
 				return;
 			}
+			if (fileOrDir.isBooksByGenreRoot()) {
+				// Display genres list
+				log.d("Show genres list");
+				mActivity.getDB().loadGenresList(fileOrDir, !mHideEmptyGenres, new ItemGroupsLoadingCallback(fileOrDir, itemToSelect));
+				return;
+			}
 			if (fileOrDir.isBooksByAuthorRoot()) {
 				// refresh authors list
 				log.d("Updating authors list");
-				mActivity.getDB().loadAuthorsList(fileOrDir, new ItemGroupsLoadingCallback(fileOrDir));
+				mActivity.getDB().loadAuthorsList(fileOrDir, new ItemGroupsLoadingCallback(fileOrDir, itemToSelect));
 				return;
 			}
 			if (fileOrDir.isBooksBySeriesRoot()) {
 				// refresh authors list
 				log.d("Updating series list");
-				mActivity.getDB().loadSeriesList(fileOrDir, new ItemGroupsLoadingCallback(fileOrDir));
+				mActivity.getDB().loadSeriesList(fileOrDir, new ItemGroupsLoadingCallback(fileOrDir, itemToSelect));
 				return;
 			}
 			if (fileOrDir.isBooksByRatingRoot()) {
 				log.d("Updating rated books list");
-				mActivity.getDB().loadBooksByRating(1, 10, new FileInfoLoadingCallback(fileOrDir));
+				mActivity.getDB().loadBooksByRating(1, 10, new FileInfoLoadingCallback(fileOrDir, itemToSelect));
 				return;
 			}
 			if (fileOrDir.isBooksByStateFinishedRoot()) {
 				log.d("Updating books by state=finished");
-				mActivity.getDB().loadBooksByState(FileInfo.STATE_FINISHED, new FileInfoLoadingCallback(fileOrDir));
+				mActivity.getDB().loadBooksByState(FileInfo.STATE_FINISHED, new FileInfoLoadingCallback(fileOrDir, itemToSelect));
 				return;
 			}
 			if (fileOrDir.isBooksByStateReadingRoot()) {
 				log.d("Updating books by state=reading");
-				mActivity.getDB().loadBooksByState(FileInfo.STATE_READING, new FileInfoLoadingCallback(fileOrDir));
+				mActivity.getDB().loadBooksByState(FileInfo.STATE_READING, new FileInfoLoadingCallback(fileOrDir, itemToSelect));
 				return;
 			}
 			if (fileOrDir.isBooksByStateToReadRoot()) {
 				log.d("Updating books by state=toRead");
-				mActivity.getDB().loadBooksByState(FileInfo.STATE_TO_READ, new FileInfoLoadingCallback(fileOrDir));
+				mActivity.getDB().loadBooksByState(FileInfo.STATE_TO_READ, new FileInfoLoadingCallback(fileOrDir, itemToSelect));
 				return;
 			}
 			if (fileOrDir.isBooksByTitleRoot()) {
 				// refresh authors list
 				log.d("Updating title list");
-				mActivity.getDB().loadTitleList(fileOrDir, new ItemGroupsLoadingCallback(fileOrDir));
+				mActivity.getDB().loadTitleList(fileOrDir, new ItemGroupsLoadingCallback(fileOrDir, itemToSelect));
+				return;
+			}
+			if (fileOrDir.isBooksByGenreDir()) {
+				log.d("Updating genres book list");
+				mActivity.getDB().loadGenresBooks(fileOrDir.getGenreCode(), !mHideEmptyGenres, new FileInfoLoadingCallback(fileOrDir, itemToSelect));
 				return;
 			}
 			if (fileOrDir.isBooksByAuthorDir()) {
 				log.d("Updating author book list");
-				mActivity.getDB().loadAuthorBooks(fileOrDir.getAuthorId(), new FileInfoLoadingCallback(fileOrDir));
+				mActivity.getDB().loadAuthorBooks(fileOrDir.getAuthorId(), new FileInfoLoadingCallback(fileOrDir, itemToSelect));
 				return;
 			}
 			if (fileOrDir.isBooksBySeriesDir()) {
 				log.d("Updating series book list");
-				mActivity.getDB().loadSeriesBooks(fileOrDir.getSeriesId(), new FileInfoLoadingCallback(fileOrDir));
+				mActivity.getDB().loadSeriesBooks(fileOrDir.getSeriesId(), new FileInfoLoadingCallback(fileOrDir, itemToSelect));
 				return;
 			}
 		} else {
@@ -889,40 +918,57 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 		final FileInfo file = fileOrDir==null || fileOrDir.isDirectory ? itemToSelect : fileOrDir;
 		final FileInfo dir = fileOrDir!=null && !fileOrDir.isDirectory ? mScanner.findParent(file, mScanner.getRoot()) : fileOrDir;
 		if ( dir!=null ) {
-			mScanner.scanDirectory(mActivity.getDB(), dir, new Runnable() {
-				public void run() {
+			if (dir.isSpecialDir()) {
+				showDirectoryInternal(dir, file);
+			} else {
+				// if previous scan is in progress, interrupt it
+				if (!mScanControl.isStopped())
+					mScanControl.stop();
+				mScanControl = new Scanner.ScanControl();
+				mScanner.scanDirectory(mActivity.getDB(), dir, () -> {
 					if (dir.allowSorting())
 						dir.sort(mSortOrder);
 					showDirectoryInternal(dir, file);
-				}
-			}, false, new Scanner.ScanControl() );
+					mActivity.setBrowserProgressStatus(true);
+				}, (scanControl) -> {
+					if (!scanControl.isStopped()) {
+						if (dir.allowSorting())
+							dir.sort(mSortOrder);
+						showDirectoryInternal(dir, file);
+					}
+					mActivity.setBrowserProgressStatus(false);
+				}, false, mScanControl);
+			}
 		} else
 			showDirectoryInternal(null, file);
 	}
 	
 	public void scanCurrentDirectoryRecursive() {
-		if (currDirectory == null)
+		if (currDirectory == null || currDirectory.isSpecialDir())
 			return;
 		log.i("scanCurrentDirectoryRecursive started");
-		final Scanner.ScanControl control = new Scanner.ScanControl(); 
-		final ProgressDialog dlg = ProgressDialog.show(mActivity, 
+		if (!mScanControl.isStopped())
+			mScanControl.stop();
+		mScanControl = new Scanner.ScanControl();
+		final ProgressDialog dlg = ProgressDialog.show(mActivity,
 				mActivity.getString(R.string.dlg_scan_title), 
 				mActivity.getString(R.string.dlg_scan_message),
-				true, true, new OnCancelListener() {
-					@Override
-					public void onCancel(DialogInterface dialog) {
-						log.i("scanCurrentDirectoryRecursive : stop handler");
-						control.stop();
-					}
-		});
-		mScanner.scanDirectory(mActivity.getDB(), currDirectory, new Runnable() {
-			@Override
-			public void run() {
-				log.i("scanCurrentDirectoryRecursive : finish handler");
-				if ( dlg.isShowing() )
-					dlg.dismiss();
+				true, true, dialog -> {
+					log.i("scanCurrentDirectoryRecursive : stop handler");
+					mScanControl.stop();
+				});
+		mScanner.scanDirectory(mActivity.getDB(), currDirectory, () -> {
+			showDirectoryInternal(currDirectory, null);
+		}, (scanControl) -> {
+			log.i("scanCurrentDirectoryRecursive : finish handler");
+			if (!scanControl.isStopped()) {
+				if (currDirectory.allowSorting())
+					currDirectory.sort(mSortOrder);
+				showDirectoryInternal(currDirectory, null);
 			}
-		}, true, control); 
+			if (dlg.isShowing())
+				dlg.dismiss();
+		}, true, mScanControl);
 	}
 
 
@@ -1011,6 +1057,7 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 			TextView field1;
 			TextView field2;
 			//TextView field3;
+			ImageView infoButton;
 			void setText( TextView view, String text )
 			{
 				if ( view==null )
@@ -1041,7 +1088,9 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 					return;
 				}
 				if ( item.isDirectory ) {
-					if (item.isBooksByAuthorRoot())
+					if (item.isBooksByGenreRoot() || item.isBooksByGenreDir())
+						image.setImageResource(Utils.resolveResourceIdByAttr(mActivity, R.attr.cr3_browser_folder_authors_drawable, R.drawable.cr3_browser_folder_authors));
+					else if (item.isBooksByAuthorRoot())
 						image.setImageResource(Utils.resolveResourceIdByAttr(mActivity, R.attr.cr3_browser_folder_authors_drawable, R.drawable.cr3_browser_folder_authors));
 					else if (item.isBooksBySeriesRoot())
 						image.setImageResource(Utils.resolveResourceIdByAttr(mActivity, R.attr.cr3_browser_folder_authors_drawable, R.drawable.cr3_browser_folder_authors));
@@ -1069,7 +1118,20 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 					
 					setText(name, title);
 
-					if ( item.isBooksByAuthorDir() ) {
+					if ( item.isBooksByGenreDir() ) {
+						if (item.tag instanceof Integer) {
+							int code = (Integer) item.tag;
+							setText(field1, "books: " + (code & FileInfo.GENRE_DATA_BOOKCOUNT_MASK));
+							if ((code & FileInfo.GENRE_DATA_INCCHILD_MASK) == FileInfo.GENRE_DATA_INCCHILD_MASK) {
+								setText(field2, mActivity.getString(R.string.including_subgenres));
+							} else {
+								setText(field2, "");
+							}
+						} else {
+							setText(field1, "");
+							setText(field2, "");
+						}
+					} else if ( item.isBooksByAuthorDir() ) {
 						int bookCount = 0;
 						if (item.fileCount() > 0)
 							bookCount = item.fileCount();
@@ -1158,6 +1220,8 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 						if (field2 != null)
 							field2.setText(Utils.formatLastPosition(mActivity, mHistory.getLastPos(item)));
 						//field3.setText(pos!=null ? formatPercent(pos.getPercent()) : null);
+						if (infoButton != null)
+							infoButton.setOnClickListener(v -> mActivity.editBookInfo(Services.getScanner().createRecentRoot(), item));
 					} 
 					
 				}
@@ -1182,14 +1246,15 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 				else
 					view = mInflater.inflate(R.layout.browser_item_book, null);
 				holder = new ViewHolder();
-				holder.image = (ImageView)view.findViewById(R.id.book_icon);
-				holder.name = (TextView)view.findViewById(R.id.book_name);
-				holder.author = (TextView)view.findViewById(R.id.book_author);
-				holder.series = (TextView)view.findViewById(R.id.book_series);
-				holder.filename = (TextView)view.findViewById(R.id.book_filename);
-				holder.field1 = (TextView)view.findViewById(R.id.browser_item_field1);
-				holder.field2 = (TextView)view.findViewById(R.id.browser_item_field2);
+				holder.image = view.findViewById(R.id.book_icon);
+				holder.name = view.findViewById(R.id.book_name);
+				holder.author = view.findViewById(R.id.book_author);
+				holder.series = view.findViewById(R.id.book_series);
+				holder.filename = view.findViewById(R.id.book_filename);
+				holder.field1 = view.findViewById(R.id.browser_item_field1);
+				holder.field2 = view.findViewById(R.id.browser_item_field2);
 				//holder.field3 = (TextView)view.findViewById(R.id.browser_item_field3);
+				holder.infoButton = view.findViewById(R.id.browser_item_button_info);
 				view.setTag(holder);
 			} else {
 				view = convertView;
@@ -1270,9 +1335,7 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 		if ( !BackgroundThread.isGUIThread() )
 			throw new IllegalStateException("showDirectoryInternal should be called from GUI thread!");
 		int index = dir!=null ? dir.getItemIndex(file) : -1;
-		if ( dir!=null && !dir.isRootDir() )
-			index++;
-		
+
 		String title = "";
 		if (dir != null) {
 			title = dir.filename;
@@ -1283,8 +1346,8 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
 		}
 		
 		mActivity.setBrowserTitle(title);
-		
-		mListView.setAdapter(currentListAdapter);
+		if (mListView.getAdapter() != currentListAdapter)
+			mListView.setAdapter(currentListAdapter);
 		currentListAdapter.notifyDataSetChanged();
 		mListView.setSelection(index);
 		mListView.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
@@ -1353,7 +1416,7 @@ public class FileBrowser extends LinearLayout implements FileInfoChangeListener
     private void setCoverSizes(int screenWidth, int screenHeight) {
     	this.screenWidth = screenWidth;
     	this.screenHeight = screenHeight;
-    	int minScreenSize = screenWidth < screenHeight ? screenWidth : screenHeight;
+    	int minScreenSize = Math.min(screenWidth, screenHeight);
     	int minh = 80;
     	int maxh = minScreenSize / 3;
     	int avgh = (minh + maxh) / 2;  
