@@ -1,17 +1,21 @@
 package org.coolreader.crengine;
 
+import android.os.Parcel;
+import android.os.Parcelable;
 import android.util.Log;
+
 import org.coolreader.R;
 import org.coolreader.plugins.OnlineStoreBook;
 
 import java.io.File;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.zip.ZipEntry;
 
-public class FileInfo {
+public class FileInfo implements Parcelable {
 
 	public final static String RECENT_DIR_TAG = "@recent";
 	public final static String SEARCH_RESULT_DIR_TAG = "@searchResults";
@@ -19,6 +23,9 @@ public class FileInfo {
 	public final static String OPDS_LIST_TAG = "@opds";
 	public final static String OPDS_DIR_PREFIX = "@opds:";
 	public final static String ONLINE_CATALOG_PLUGIN_PREFIX = "@plugin:";
+	public final static String GENRES_TAG = "@genresRoot";
+	public final static String GENRES_GROUP_PREFIX = "@genresGroup:";
+	public final static String GENRES_PREFIX = "@genre:";
 	public final static String AUTHORS_TAG = "@authorsRoot";
 	public final static String AUTHOR_GROUP_PREFIX = "@authorGroup:";
 	public final static String AUTHOR_PREFIX = "@author:";
@@ -40,16 +47,18 @@ public class FileInfo {
 	public String authors; // authors, delimited with '|'
 	public String series; // series name w/o number
 	public int seriesNumber; // number of book inside series
+	public String genres; // genre codes, delimited with '|'
 	public String path; // path to directory where file or archive is located
 	public String filename; // file name w/o path for normal file, with optional path for file inside archive 
 	public String pathname; // full path+arcname+filename
 	public String arcname; // archive file name w/o path
 	public String language; // document language
+	public String description;	// book description
 	public String username; // username for online catalogs
 	public String password; // password for online catalogs
 	public DocumentFormat format;
-	public int size; // full file size
-	public int arcsize; // compressed size
+	public long size; // full file size
+	public long arcsize; // compressed size
 	public long createTime;
 	public long lastAccessTime;
 	public int flags;
@@ -57,6 +66,9 @@ public class FileInfo {
 	public boolean isDirectory;
 	public boolean isListed;
 	public boolean isScanned;
+	public long crc32;
+	public int domVersion;
+	public int blockRenderingFlags;
 	public FileInfo parent; // parent item
 	public Object tag; // some additional information
 	
@@ -93,6 +105,26 @@ public class FileInfo {
     public static final int TYPE_FS_ROOT = 1;
     public static final int TYPE_DOWNLOAD_DIR = 2;
 
+    // bits 26..29 - profile id (0..15 max)
+	public static final int PROFILE_ID_SHIFT = 26;
+	public static final int PROFILE_ID_MASK = 0x0F;
+
+	// bitmask for field 'tag' when obtained genres list as special folders
+	public static final int GENRE_DATA_INCCHILD_MASK = 0x80000000;
+	public static final int GENRE_DATA_BOOKCOUNT_MASK = 0x00FFFFFF;
+
+	public static final Creator<FileInfo> CREATOR = new Creator<FileInfo>() {
+		@Override
+		public FileInfo createFromParcel(Parcel in) {
+			return new FileInfo(in);
+		}
+
+		@Override
+		public FileInfo[] newArray(int size) {
+			return new FileInfo[size];
+		}
+	};
+
 	/**
 	 * Get book reading state. 
 	 * @return reading state (one of STATE_XXX constants)
@@ -145,11 +177,8 @@ public class FileInfo {
 	 * To separate archive name from file name inside archive.
 	 */
 	public static final String ARC_SEPARATOR = "@/";
-	
-	public static final int PROFILE_ID_SHIFT = 16;
-	public static final int PROFILE_ID_MASK = 0x0F;
-	
-	
+
+
 	public void setFlag( int flag, boolean value ) {
 		flags = flags & (~flag) | (value? flag : 0);
 	}
@@ -205,7 +234,43 @@ public class FileInfo {
 		}
 		return res;
 	}
-	
+
+	protected FileInfo(Parcel in) {
+		if (in.readByte() == 0) {
+			id = null;
+		} else {
+			id = in.readLong();
+		}
+		title = in.readString();
+		authors = in.readString();
+		series = in.readString();
+		seriesNumber = in.readInt();
+		genres = in.readString();
+		path = in.readString();
+		filename = in.readString();
+		pathname = in.readString();
+		arcname = in.readString();
+		language = in.readString();
+		description = in.readString();
+		username = in.readString();
+		password = in.readString();
+		size = in.readLong();
+		arcsize = in.readLong();
+		createTime = in.readLong();
+		lastAccessTime = in.readLong();
+		flags = in.readInt();
+		isArchive = in.readByte() != 0;
+		isDirectory = in.readByte() != 0;
+		isListed = in.readByte() != 0;
+		isScanned = in.readByte() != 0;
+		crc32 = in.readLong();
+		domVersion = in.readInt();
+		blockRenderingFlags = in.readInt();
+		parent = in.readParcelable(FileInfo.class.getClassLoader());
+		files = in.createTypedArrayList(FileInfo.CREATOR);
+		dirs = in.createTypedArrayList(FileInfo.CREATOR);
+	}
+
 	public FileInfo( String pathName )
 	{
 		String[] parts = splitArcName( pathName );
@@ -219,7 +284,7 @@ public class FileInfo {
 			path = f.getPath();
 			File arc = new File(arcname);
 			if (arc.isFile() && arc.exists()) {
-				arcsize = (int)arc.length();
+				arcsize = arc.length();
 				isArchive = true;
 				try {
 					//ZipFile zip = new ZipFile(new File(arcname));
@@ -233,9 +298,11 @@ public class FileInfo {
 							filename = itemf.getName();
 							path = itemf.getPath();
 							format = DocumentFormat.byExtension(name);
-							size = (int)entry.getSize();
-							arcsize = (int)entry.getCompressedSize();
+							size = entry.getSize();
+							//arcsize = entry.getCompressedSize();
 							createTime = entry.getTime();
+							domVersion = Engine.DOM_VERSION_CURRENT;
+							blockRenderingFlags = Engine.BLOCK_RENDERING_FLAGS_WEB;
 							break;
 						}
 					}
@@ -263,13 +330,18 @@ public class FileInfo {
 			pathname = f.getAbsolutePath();
 			format = fmt;
 			createTime = f.lastModified();
-			size = (int)f.length();
+			size = f.length();
+			domVersion = Engine.DOM_VERSION_CURRENT;
+			blockRenderingFlags = Engine.BLOCK_RENDERING_FLAGS_WEB;
 		} else {
 			filename = f.getName();
 			path = f.getParent();
 			pathname = f.getAbsolutePath();
 			isDirectory = true;
 		}
+		File parent_ = f.getParentFile();
+		if (null != parent_)
+			parent = new FileInfo(parent_);
 	}
 	
 	public FileInfo( File f )
@@ -279,6 +351,8 @@ public class FileInfo {
 	
 	public FileInfo()
 	{
+		domVersion = Engine.DOM_VERSION_CURRENT;
+		blockRenderingFlags = Engine.BLOCK_RENDERING_FLAGS_WEB;
 	}
 
 	/// doesn't copy parent and children
@@ -306,8 +380,13 @@ public class FileInfo {
 		createTime = v.createTime;
 		lastAccessTime = v.lastAccessTime;
 		language = v.language;
+		genres = v.genres;
+		description = v.description;
 		username = v.username;
 		password = v.password;
+		crc32 = v.crc32;
+		domVersion = v.domVersion;
+		blockRenderingFlags = v.blockRenderingFlags;
 		id = v.id;
 	}
 	
@@ -390,7 +469,12 @@ public class FileInfo {
 	{
 		return SEARCH_SHORTCUT_TAG.equals(pathname);
 	}
-	
+
+	public boolean isBooksByGenreRoot()
+	{
+		return GENRES_TAG.equals(pathname);
+	}
+
 	public boolean isBooksByAuthorRoot()
 	{
 		return AUTHORS_TAG.equals(pathname);
@@ -425,7 +509,12 @@ public class FileInfo {
 	{
 		return TITLE_TAG.equals(pathname);
 	}
-	
+
+	public boolean isBooksByGenreDir()
+	{
+		return pathname!=null && pathname.startsWith(GENRES_PREFIX);
+	}
+
 	public boolean isBooksByAuthorDir()
 	{
 		return pathname!=null && pathname.startsWith(AUTHOR_PREFIX);
@@ -447,6 +536,13 @@ public class FileInfo {
 		return parent.isOnSDCard();
 	}
 
+	public String getGenreCode() {
+		if (pathname.startsWith(GENRES_PREFIX)) {
+			return pathname.substring(GENRES_PREFIX.length());
+		}
+		return "";
+	}
+
 	public long getAuthorId()
 	{
 		if (!isBooksByAuthorDir())
@@ -574,6 +670,26 @@ public class FileInfo {
 		dirs = null;
 		addItems( items );
 	}
+	public boolean updateItem( FileInfo item ) {
+		if (null != dirs) {
+			for (FileInfo dir : dirs) {
+				if (dir.pathNameEquals(item)) {
+					dir.assign(item);
+					dir.setItems(item);
+					return true;
+				}
+			}
+		}
+		if (null != files) {
+			for (FileInfo file : files) {
+				if (file.pathNameEquals(item)) {
+					file.assign(item);
+					return true;
+				}
+			}
+		}
+		return false;
+	}
 	public boolean isEmpty()
 	{
 		return fileCount()==0 && dirCount()==0;
@@ -699,15 +815,21 @@ public class FileInfo {
 
 	public void setItems(FileInfo copyFrom)
 	{
+		if (this == copyFrom)
+			return;
 		clear();
 		for (int i=0; i<copyFrom.fileCount(); i++) {
-			addFile(copyFrom.getFile(i));
-			copyFrom.getFile(i).parent = this;
+			FileInfo file = copyFrom.getFile(i);
+			file.parent = this;
+			addFile(file);
 		}
 		for (int i=0; i<copyFrom.dirCount(); i++) {
-			addDir(copyFrom.getDir(i));
-			copyFrom.getDir(i).parent = this;
+			FileInfo dir = copyFrom.getDir(i);
+			dir.parent = this;
+			addDir(dir);
 		}
+		isListed = copyFrom.isListed;
+		isScanned = copyFrom.isScanned;
 	}
 
 	public void setItems(Collection<FileInfo> list)
@@ -722,17 +844,22 @@ public class FileInfo {
 				addFile(item);
 			item.parent = this;
 		}
+		isListed = true;
 	}
 
-	public void removeEmptyDirs()
+	public boolean removeEmptyDirs()
 	{
 		if ( parent==null || pathname.startsWith("@") || !isListed || dirs==null )
-			return;
+			return false;
+		boolean removed = false;
 		for ( int i=dirCount()-1; i>=0; i-- ) {
 			FileInfo dir = getDir(i);
-			if ( dir.isListed && dir.dirCount() == 0 && dir.fileCount() == 0)
+			if ( dir.isListed && dir.dirCount() == 0 && dir.fileCount() == 0) {
 				dirs.remove(i);
+				removed = true;
+			}
 		}
+		return removed;
 	}
 	
 	public void removeChild( FileInfo item )
@@ -786,7 +913,7 @@ public class FileInfo {
 		}
 		return false;
 	}
-	
+
 	public boolean fileExists()
 	{
 		if (isDirectory)
@@ -906,7 +1033,50 @@ public class FileInfo {
 		dirs = null;
 		files = null;
 	}
-	
+
+	@Override
+	public int describeContents() {
+		return 0;
+	}
+
+	@Override
+	public void writeToParcel(Parcel dest, int flags) {
+		if (id == null) {
+			dest.writeByte((byte) 0);
+		} else {
+			dest.writeByte((byte) 1);
+			dest.writeLong(id);
+		}
+		dest.writeString(title);
+		dest.writeString(authors);
+		dest.writeString(series);
+		dest.writeInt(seriesNumber);
+		dest.writeString(genres);
+		dest.writeString(path);
+		dest.writeString(filename);
+		dest.writeString(pathname);
+		dest.writeString(arcname);
+		dest.writeString(language);
+		dest.writeString(description);
+		dest.writeString(username);
+		dest.writeString(password);
+		dest.writeLong(size);
+		dest.writeLong(arcsize);
+		dest.writeLong(createTime);
+		dest.writeLong(lastAccessTime);
+		dest.writeInt(flags);
+		dest.writeByte((byte) (isArchive ? 1 : 0));
+		dest.writeByte((byte) (isDirectory ? 1 : 0));
+		dest.writeByte((byte) (isListed ? 1 : 0));
+		dest.writeByte((byte) (isScanned ? 1 : 0));
+		dest.writeLong(crc32);
+		dest.writeInt(domVersion);
+		dest.writeInt(blockRenderingFlags);
+		dest.writeParcelable(parent, flags);
+		dest.writeTypedList(files);
+		dest.writeTypedList(dirs);
+	}
+
 	public static enum SortOrder {
 		FILENAME(R.string.mi_book_sort_order_filename, new Comparator<FileInfo>() {
 			public int compare( FileInfo f1, FileInfo f2 )
@@ -1075,7 +1245,7 @@ public class FileInfo {
 		final int prime = 31;
 		int result = 1;
 		result = prime * result + ((arcname == null) ? 0 : arcname.hashCode());
-		result = prime * result + arcsize;
+		result = prime * result + (int)arcsize;
 		result = prime * result + ((authors == null) ? 0 : authors.hashCode());
 		result = prime * result + (int) (createTime ^ (createTime >>> 32));
 		result = prime * result + ((dirs == null) ? 0 : dirs.hashCode());
@@ -1098,7 +1268,7 @@ public class FileInfo {
 				+ ((pathname == null) ? 0 : pathname.hashCode());
 		result = prime * result + ((series == null) ? 0 : series.hashCode());
 		result = prime * result + seriesNumber;
-		result = prime * result + size;
+		result = prime * result + (int)size;
 		result = prime * result + ((title == null) ? 0 : title.hashCode());
 		return result;
 	}
@@ -1158,6 +1328,17 @@ public class FileInfo {
 				return false;
 		} else if (!language.equals(other.language))
 			return false;
+		// do not compare genres of books, because in the absence of certain genres in the handbook,
+		// the 'genres' field obtained from the database will not be equal to the field obtained when parsing the book file.
+		/*
+		if (!eqGenre(genres, other.genres))
+			return false;
+		*/
+		if (description == null) {
+			if (other.description != null)
+				return false;
+		} else if (!description.equals(other.description))
+			return false;
 		if (lastAccessTime != other.lastAccessTime)
 			return false;
 		if (parent == null) {
@@ -1176,9 +1357,9 @@ public class FileInfo {
 		} else if (!pathname.equals(other.pathname))
 			return false;
 		if (series == null) {
-			if (other.series != null)
+			if (other.series != null && other.series.length() != 0)
 				return false;
-		} else if (!series.equals(other.series))
+		} else if (!series.equals(other.series) && !(series.length() == 0 && other.series == null))
 			return false;
 		if (seriesNumber != other.seriesNumber)
 			return false;
@@ -1189,7 +1370,88 @@ public class FileInfo {
 				return false;
 		} else if (!title.equals(other.title))
 			return false;
-		return true;
+		if (domVersion != other.domVersion)
+			return false;
+		if (blockRenderingFlags != other.blockRenderingFlags)
+			return false;
+		// Base check: fingerprint (for now only crc32)
+		return crc32 == other.crc32;
+	}
+
+	public boolean baseEquals(FileInfo other) {
+		if (this == other)
+			return true;
+		if (other == null)
+			return false;
+		if (arcsize != other.arcsize)
+			return false;
+		if (authors == null) {
+			if (other.authors != null)
+				return false;
+		} else if (!authors.equals(other.authors))
+			return false;
+		// TODO: potentially filename may not match. Perhaps we need to remove this check.
+		if (filename == null) {
+			if (other.filename != null)
+				return false;
+		} else if (!filename.equals(other.filename))
+			return false;
+		if (format != other.format)
+			return false;
+		// TODO: potentially isArchive may not match. Perhaps we need to remove this check.
+		if (isArchive != other.isArchive)
+			return false;
+		if (isDirectory != other.isDirectory)
+			return false;
+		if (language == null) {
+			if (other.language != null)
+				return false;
+		} else if (!language.equals(other.language))
+			return false;
+		// do not compare genres of books, because in the absence of certain genres in the handbook,
+		// the 'genres' field obtained from the database will not be equal to the field obtained when parsing the book file.
+		/*
+		if (!eqGenre(genres, other.genres))
+			return false;
+		*/
+		if (description == null) {
+			if (other.description != null)
+				return false;
+		} else if (!description.equals(other.description))
+			return false;
+		if (series == null) {
+			if (other.series != null && other.series.length() != 0)
+				return false;
+		} else if (!series.equals(other.series) && !(series.length() == 0 && other.series == null))
+			return false;
+		if (seriesNumber != other.seriesNumber)
+			return false;
+		if (size != other.size)
+			return false;
+		if (title == null) {
+			if (other.title != null)
+				return false;
+		} else if (!title.equals(other.title))
+			return false;
+		// Base check: fingerprint (for now only crc32)
+		return crc32 == other.crc32;
+	}
+
+	private static boolean eqGenre(String g1, String g2) {
+		if (g1 == null) {
+			if (g2 != null && g2.length() != 0)
+				return false;
+		}
+		if (g1.equals(g2))
+			return true;
+		String[] g1_array = g1.split("\\|");
+		String[] g2_array = g2.split("\\|");
+		if (g1_array.length == g2_array.length) {
+			Arrays.sort(g1_array);
+			Arrays.sort(g2_array);
+			return Arrays.equals(g1_array, g2_array);
+		}
+		return false;
 	}
 
 	@Override
