@@ -1,6 +1,13 @@
 package org.coolreader.crengine;
 
 import android.annotation.SuppressLint;
+
+import org.coolreader.CoolReader;
+import org.coolreader.crengine.Engine.DelayedProgress;
+import org.xml.sax.Attributes;
+import org.xml.sax.SAXException;
+import org.xml.sax.helpers.DefaultHandler;
+
 import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.FileOutputStream;
@@ -21,24 +28,18 @@ import java.util.Collection;
 import java.util.HashSet;
 import java.util.Stack;
 import java.util.concurrent.Callable;
-import javax.net.ssl.HostnameVerifier;
+
 import javax.net.ssl.HttpsURLConnection;
 import javax.net.ssl.SSLContext;
-import javax.net.ssl.SSLSession;
 import javax.net.ssl.TrustManager;
 import javax.net.ssl.X509TrustManager;
 import javax.xml.parsers.SAXParser;
 import javax.xml.parsers.SAXParserFactory;
-import org.coolreader.CoolReader;
-import org.coolreader.crengine.Engine.DelayedProgress;
-import org.xml.sax.Attributes;
-import org.xml.sax.SAXException;
-import org.xml.sax.helpers.DefaultHandler;
 
 @SuppressLint("SimpleDateFormat")
 public class OPDSUtil {
 
-	public static final boolean EXTENDED_LOG = false;
+	public static final boolean EXTENDED_LOG = false; // set to false for production
     public static final int CONNECT_TIMEOUT = 60000;
     public static final int READ_TIMEOUT = 60000;
 	/*
@@ -220,6 +221,7 @@ xml:base="http://lib.ololo.cc/opds/">
 		private AuthorInfo authorInfo;
 		private boolean insideFeed;
 		private boolean insideEntry;
+		private boolean insideEntryTitle;
 		//private boolean singleEntry;
 		private int level = 0;
 		//2011-05-31T10:28:22+04:00
@@ -277,15 +279,16 @@ xml:base="http://lib.ololo.cc/opds/">
 					else
 						docInfo.updated = ts;
 				} else if ( "title".equals(currentElement) ) {
-					if ( !insideEntry )
+					if ( !insideEntry ) {
 						docInfo.title = s;
-					else
+					} else {
 						entryInfo.title = entryInfo.title + s;
+					}
 				} else if ( "summary".equals(currentElement) ) {
 					if ( insideEntry )
 						entryInfo.summary = entryInfo.summary + s;
 				} else if ( "name".equals(currentElement) ) {
-					if ( authorInfo!=null )
+					if ( authorInfo != null )
 						authorInfo.name = s;
 				} else if ( "uri".equals(currentElement) ) {
 					if ( authorInfo!=null )
@@ -310,6 +313,10 @@ xml:base="http://lib.ololo.cc/opds/">
 				} else if ( "language".equals(currentElement) ) {
 					if ( !insideEntry )
 						docInfo.language = s;
+				} else if ( insideEntryTitle ) {
+					if (entryInfo.title.length() > 0)
+						entryInfo.title = entryInfo.title + " ";
+					entryInfo.title = entryInfo.title + s;
 				}
 			}
 		}
@@ -365,7 +372,9 @@ xml:base="http://lib.ololo.cc/opds/">
 			} else if ( "updated".equals(localName) ) {
 				
 			} else if ( "title".equals(localName) ) {
-				
+				insideEntryTitle = insideEntry;
+			} else if ( "name".equals(localName) ) {
+
 			} else if ( "link".equals(localName) ) {
 				LinkInfo link = new LinkInfo(url, attributes);
 				if ( link.isValid() && insideFeed ) {
@@ -377,6 +386,11 @@ xml:base="http://lib.ololo.cc/opds/">
 							if ( link.type.startsWith("application/atom+xml") ) {
 								if (entryInfo.link == null || !entryInfo.link.type.startsWith("application/atom+xml"))
 									entryInfo.link = link;
+							} else if ( "http://opds-spec.org/cover".equals(link.rel) && "image/jpeg".equals(link.type)) {
+								entryInfo.icon = link.href;
+							} else if ( "http://opds-spec.org/thumbnail".equals(link.rel) && "image/jpeg".equals(link.type)) {
+								if (entryInfo.icon == null)
+									entryInfo.icon = link.href;
 							} else if (priority>0 && (entryInfo.link==null || entryInfo.link.getPriority()<priority)) {
 								entryInfo.link = link;
 							}
@@ -405,6 +419,8 @@ xml:base="http://lib.ololo.cc/opds/">
 			//String currentElement = elements.peek();
 			if ( insideFeed && "feed".equals(localName) ) {
 				insideFeed = false;
+			} else if ( "title".equals(localName) ) {
+				insideEntryTitle = false;
 			} else if ( "entry".equals(localName) ) {
 				if ( !insideFeed || !insideEntry )
 					throw new SAXException("unexpected element " + localName);
@@ -462,18 +478,19 @@ xml:base="http://lib.ololo.cc/opds/">
 			if ( totalSize>0 )
 				progressMessage = progressMessage + " (" + totalSize + ")";
 		}
+		// call in GUI thread only!
+		private void hideProgress() {
+			if ( progressShown && Services.getEngine() != null)
+				Services.getEngine().hideProgress();
+			if ( delayedProgress != null ) {
+				delayedProgress.cancel();
+				delayedProgress.hide();
+			}
+		}
 		private void onError(final String msg) {
-			BackgroundThread.instance().executeGUI(new Runnable() {
-				@Override
-				public void run() {
-					if ( delayedProgress!=null ) {
-						delayedProgress.cancel();
-						delayedProgress.hide();
-					}
-					if (Services.getEngine() != null)
-						Services.getEngine().hideProgress();
-					callback.onError(msg);
-				}
+			BackgroundThread.instance().executeGUI(() -> {
+				hideProgress();
+				callback.onError(msg);
 			});
 		}
 		private void parseFeed( InputStream is ) throws Exception {
@@ -626,10 +643,8 @@ xml:base="http://lib.ololo.cc/opds/">
 			L.d("Download started: " + outFile.getAbsolutePath());
 //			long lastTs = System.currentTimeMillis(); 
 //			int lastPercent = -1;
-			FileOutputStream os = null;
 			boolean success = false;
-			try {
-				os = new FileOutputStream(outFile);
+			try (FileOutputStream os = new FileOutputStream(outFile)) {
 				byte[] buf = new byte[16384];
 				int totalWritten = 0;
 				while (totalWritten<contentLength || contentLength==-1) {
@@ -652,8 +667,6 @@ xml:base="http://lib.ololo.cc/opds/">
 				}
 				success = true;
 			} finally {
-				if ( os!=null )
-					os.close();
 				if ( !success ) {
 					if ( outFile.exists() && outFile.isFile() ) {
 						L.w("deleting unsuccessully downloaded file " + outFile);
@@ -662,12 +675,7 @@ xml:base="http://lib.ololo.cc/opds/">
 				}
 			}
 			L.d("Download finished");
-			BackgroundThread.instance().executeGUI(new Runnable() {
-				@Override
-				public void run() {
-					callback.onDownloadEnd(type, url, outFile);
-				}
-			});
+			BackgroundThread.instance().executeGUI(() -> callback.onDownloadEnd(type, url, outFile));
 		}
 		public static int findSubstring( byte[]buf, String str ) {
 			for ( int i=0; i<buf.length-str.length(); i++ ) {
@@ -686,7 +694,7 @@ xml:base="http://lib.ololo.cc/opds/">
 		public static String encodePassword(String username, String password) {
 			return Base64.encodeToString((username + ":" + password).getBytes(), Base64.NO_WRAP);
 		}
-		
+
 		public void runInternal() {
 			connection = null;
 			
@@ -699,8 +707,11 @@ xml:base="http://lib.ololo.cc/opds/">
 					setProgressMessage( url.toString(), -1 );
 					visited.add(url.toString());
 					long startTimeStamp = System.currentTimeMillis();
-					if (!partialDownloadCompleted)
+					if (!partialDownloadCompleted) {
+						if (delayedProgress != null)
+							delayedProgress.cancel();
 						delayedProgress = Services.getEngine().showProgressDelayed(0, progressMessage, PROGRESS_DELAY_MILLIS);
+					}
 					URL newURL = url;
 					boolean useOrobotProxy = false;
 					String host = url.getHost();
@@ -749,12 +760,7 @@ xml:base="http://lib.ololo.cc/opds/">
 	                    sc.init(null, trustAllCerts, new java.security.SecureRandom());
 	                    HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
 	                    
-	                	https.setHostnameVerifier(new HostnameVerifier() {
-						    @Override
-						    public boolean verify(String arg0, SSLSession arg1) {
-							    return true;
-						    }
-					    });
+	                	https.setHostnameVerifier((arg0, arg1) -> true);
 					}
 					if ( !(conn instanceof HttpURLConnection) ) {
 						onError("Only HTTP supported");
@@ -794,7 +800,24 @@ xml:base="http://lib.ololo.cc/opds/">
 					
 					response = connection.getResponseCode();
 					if (EXTENDED_LOG) L.d("Response: " + response);
-					if ( response!=200 ) {
+					if ( response == 301 || response == 302 || response == 307 || response == 303 ) {
+						// redirects
+						String redirect = connection.getHeaderField("Location");
+						if (null == redirect) {
+							onError("Invalid redirect " + response);
+							return;
+						}
+						L.d("continue with next part: " + url);
+						url = new URL(redirect);
+						if (visited.contains(url.toString())) {
+							onError("Duplicate redirect " + url);
+							return;
+						}
+						loadNext = true;
+						L.d("Response " + response + ": redirect to " + url);
+						continue;
+					}
+					if ( response != 200 ) {
 						onError("Error " + response);
 						return;
 					}
@@ -852,8 +875,7 @@ xml:base="http://lib.ololo.cc/opds/">
 							fileName = defaultFileName;
 						L.d("Downloading book: " + contentEncoding);
 						downloadBook( contentType, url.toString(), is, contentLen, fileName, isZip );
-						if ( progressShown )
-							Services.getEngine().hideProgress();
+						hideProgress();
 						loadNext = false;
 						itemsLoadedPartially = false;
 					}
@@ -878,40 +900,33 @@ xml:base="http://lib.ololo.cc/opds/">
 					// partially loaded
 					if ( progressShown )
 						Services.getEngine().hideProgress();
-					final ArrayList<EntryInfo> entries = new ArrayList<EntryInfo>();
-					entries.addAll(handler.entries);
-					BackgroundThread.instance().executeGUI(new Runnable() {
-						@Override
-						public void run() {
-							L.d("Parsing is partially. " + handler.entries.size() + " entries found -- updating view");
-							if (!callback.onEntries(handler.docInfo, entries))
-								cancel();
-						}
+					final ArrayList<EntryInfo> entries = new ArrayList<>(handler.entries);
+					BackgroundThread.instance().executeGUI(() -> {
+						L.d("Parsing is partially. " + handler.entries.size() + " entries found -- updating view");
+						if (!callback.onEntries(handler.docInfo, entries))
+							cancel();
 					});
 				}
 			} while (loadNext && !cancelled);
-			if ( progressShown )
-				Services.getEngine().hideProgress();
-			if (itemsLoadedPartially && !cancelled)
-				BackgroundThread.instance().executeGUI(new Runnable() {
-					@Override
-					public void run() {
-						L.d("Parsing is finished successfully. " + handler.entries.size() + " entries found");
-						if (!callback.onFinish(handler.docInfo, handler.entries))
-							cancel();
-					}
+			if (delayedProgress != null)
+				delayedProgress.cancel();
+			hideProgress();
+			if (itemsLoadedPartially && !cancelled) {
+				BackgroundThread.instance().executeGUI(() -> {
+					L.d("Parsing is finished successfully. " + handler.entries.size() + " entries found");
+					hideProgress();
+					if (!callback.onFinish(handler.docInfo, handler.entries))
+						cancel();
 				});
+			}
 		}
 
 		public void run() {
-			BackgroundThread.instance().postBackground(new Runnable() {
-				@Override
-				public void run() {
-					try {
-						runInternal();
-					} catch ( Exception e ) {
-						L.e("exception while opening OPDS", e);
-					}
+			BackgroundThread.instance().postBackground(() -> {
+				try {
+					runInternal();
+				} catch ( Exception e ) {
+					L.e("exception while opening OPDS", e);
 				}
 			});
 		}
