@@ -1,30 +1,42 @@
 package org.coolreader.db;
 
-import android.app.Service;
 import android.content.Intent;
 import android.os.Binder;
 import android.os.Environment;
 import android.os.Handler;
 import android.os.IBinder;
 import android.util.Log;
-import org.coolreader.crengine.*;
+
+import org.coolreader.crengine.BookInfo;
+import org.coolreader.crengine.Bookmark;
+import org.coolreader.crengine.DeviceInfo;
+import org.coolreader.crengine.Engine;
+import org.coolreader.crengine.FileInfo;
+import org.coolreader.crengine.L;
+import org.coolreader.crengine.Logger;
+import org.coolreader.crengine.MountPathCorrector;
+import org.coolreader.crengine.Scanner;
+import org.coolreader.crengine.Utils;
 
 import java.io.File;
 import java.util.ArrayList;
 import java.util.Collection;
 
-public class CRDBService extends Service {
+public class CRDBService extends BaseService {
 	public static final Logger log = L.create("db");
 	public static final Logger vlog = L.create("db", Log.ASSERT);
 
     private MainDB mainDB = new MainDB();
     private CoverDB coverDB = new CoverDB();
-	
+
+	public CRDBService() {
+		super("crdb");
+	}
+
     @Override
     public void onCreate() {
     	log.i("onCreate()");
-    	mThread = new ServiceThread("crdb");
-    	mThread.start();
+    	super.onCreate();
     	execTask(new OpenDatabaseTask());
     }
 
@@ -44,7 +56,7 @@ public class CRDBService extends Service {
     public void onDestroy() {
     	log.i("onDestroy()");
     	execTask(new CloseDatabaseTask());
-    	mThread.stop(5000);
+    	super.onDestroy();
     }
 
     private File getDatabaseDir() {
@@ -174,7 +186,7 @@ public class CRDBService extends Service {
     }
 
     public static class FileInfoCache {
-    	private ArrayList<FileInfo> list = new ArrayList<FileInfo>();
+    	private ArrayList<FileInfo> list = new ArrayList<>();
     	public void add(FileInfo item) {
     		list.add(item);
     	}
@@ -224,14 +236,9 @@ public class CRDBService extends Service {
 		execTask(new Task("loadOPDSCatalogs") {
 			@Override
 			public void work() {
-				final ArrayList<FileInfo> list = new ArrayList<FileInfo>(); 
+				final ArrayList<FileInfo> list = new ArrayList<>();
 				mainDB.loadOPDSCatalogs(list);
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onOPDSCatalogsLoaded(list);
-					}
-				});
+				sendTask(handler, () -> callback.onOPDSCatalogsLoaded(list));
 			}
 		});
 	}
@@ -241,17 +248,11 @@ public class CRDBService extends Service {
 			@Override
 			public void work() {
 				final ArrayList<String> list = mainDB.loadSearchHistory(book);
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onSearchHistoryLoaded(list);
-					}
-				});
+				sendTask(handler, () -> callback.onSearchHistoryLoaded(list));
 			}
 		});
 	}
 
-
 	public void removeOPDSCatalog(final Long id) {
 		execTask(new Task("removeOPDSCatalog") {
 			@Override
@@ -286,12 +287,7 @@ public class CRDBService extends Service {
 			@Override
 			public void work() {
 				final byte[] data = coverDB.loadBookCoverpage(fileInfo.getPathName());
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onCoverpageLoaded(fileInfo, data);
-					}
-				});
+				sendTask(handler, () -> callback.onCoverpageLoaded(fileInfo, data));
 			}
 		});
 	}
@@ -328,19 +324,25 @@ public class CRDBService extends Service {
     public interface BookSearchCallback {
     	void onBooksFound(ArrayList<FileInfo> fileList);
     }
-    
+
+	public void loadGenresList(FileInfo parent, boolean showEmptyGenres, final ItemGroupsLoadingCallback callback, final Handler handler) {
+		final FileInfo p = new FileInfo(parent);
+		execTask(new Task("loadGenresList") {
+			@Override
+			public void work() {
+				mainDB.loadGenresList(p, showEmptyGenres);
+				sendTask(handler, () -> callback.onItemGroupsLoaded(p));
+			}
+		});
+	}
+
 	public void loadAuthorsList(FileInfo parent, final ItemGroupsLoadingCallback callback, final Handler handler) {
 		final FileInfo p = new FileInfo(parent); 
 		execTask(new Task("loadAuthorsList") {
 			@Override
 			public void work() {
 				mainDB.loadAuthorsList(p);
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onItemGroupsLoaded(p);
-					}
-				});
+				sendTask(handler, () -> callback.onItemGroupsLoaded(p));
 			}
 		});
 	}
@@ -351,12 +353,7 @@ public class CRDBService extends Service {
 			@Override
 			public void work() {
 				mainDB.loadSeriesList(p);
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onItemGroupsLoaded(p);
-					}
-				});
+				sendTask(handler, () -> callback.onItemGroupsLoaded(p));
 			}
 		});
 	}
@@ -367,12 +364,17 @@ public class CRDBService extends Service {
 			@Override
 			public void work() {
 				mainDB.loadTitleList(p);
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onItemGroupsLoaded(p);
-					}
-				});
+				sendTask(handler, () -> callback.onItemGroupsLoaded(p));
+			}
+		});
+	}
+
+	public void findGenresBooks(final String genreCode, boolean showEmptyGenres, final FileInfoLoadingCallback callback, final Handler handler) {
+		execTask(new Task("findGenresBooks") {
+			@Override
+			public void work() {
+				final ArrayList<FileInfo> list = mainDB.findByGenre(genreCode, showEmptyGenres);
+				sendTask(handler, () -> callback.onFileInfoListLoaded(list));
 			}
 		});
 	}
@@ -381,14 +383,9 @@ public class CRDBService extends Service {
 		execTask(new Task("findAuthorBooks") {
 			@Override
 			public void work() {
-				final ArrayList<FileInfo> list = new ArrayList<FileInfo>();
+				final ArrayList<FileInfo> list = new ArrayList<>();
 				mainDB.findAuthorBooks(list, authorId);
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onFileInfoListLoaded(list);
-					}
-				});
+				sendTask(handler, () -> callback.onFileInfoListLoaded(list));
 			}
 		});
 	}
@@ -397,14 +394,9 @@ public class CRDBService extends Service {
 		execTask(new Task("findSeriesBooks") {
 			@Override
 			public void work() {
-				final ArrayList<FileInfo> list = new ArrayList<FileInfo>();
+				final ArrayList<FileInfo> list = new ArrayList<>();
 				mainDB.findSeriesBooks(list, seriesId);
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onFileInfoListLoaded(list);
-					}
-				});
+				sendTask(handler, () -> callback.onFileInfoListLoaded(list));
 			}
 		});
 	}
@@ -413,14 +405,9 @@ public class CRDBService extends Service {
 		execTask(new Task("findBooksByRating") {
 			@Override
 			public void work() {
-				final ArrayList<FileInfo> list = new ArrayList<FileInfo>();
+				final ArrayList<FileInfo> list = new ArrayList<>();
 				mainDB.findBooksByRating(list, minRate, maxRate);
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onFileInfoListLoaded(list);
-					}
-				});
+				sendTask(handler, () -> callback.onFileInfoListLoaded(list));
 			}
 		});
 	}
@@ -429,14 +416,9 @@ public class CRDBService extends Service {
 		execTask(new Task("findBooksByState") {
 			@Override
 			public void work() {
-				final ArrayList<FileInfo> list = new ArrayList<FileInfo>();
+				final ArrayList<FileInfo> list = new ArrayList<>();
 				mainDB.findBooksByState(list, state);
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onFileInfoListLoaded(list);
-					}
-				});
+				sendTask(handler, () -> callback.onFileInfoListLoaded(list));
 			}
 		});
 	}
@@ -446,12 +428,7 @@ public class CRDBService extends Service {
 			@Override
 			public void work() {
 				final ArrayList<BookInfo> list = mainDB.loadRecentBooks(maxCount);
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onRecentBooksListLoaded(list);
-					}
-				});
+				sendTask(handler, () -> callback.onRecentBooksListLoaded(list));
 			}
 		});
 	}
@@ -460,27 +437,27 @@ public class CRDBService extends Service {
 		execTask(new Task("sync") {
 			@Override
 			public void work() {
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.run();
-					}
-				});
+				sendTask(handler, callback);
 			}
 		});
 	}
 	
-	public void findByPatterns(final int maxCount, final String author, final String title, final String series, final String filename, final BookSearchCallback callback, final Handler handler) {
+	public void findByPatterns(final int maxCount, final String authors, final String title, final String series, final String filename, final BookSearchCallback callback, final Handler handler) {
 		execTask(new Task("findByPatterns") {
 			@Override
 			public void work() {
-				final ArrayList<FileInfo> list = mainDB.findByPatterns(maxCount, author, title, series, filename);
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onBooksFound(list);
-					}
-				});
+				final ArrayList<FileInfo> list = mainDB.findByPatterns(maxCount, authors, title, series, filename);
+				sendTask(handler, () -> callback.onBooksFound(list));
+			}
+		});
+	}
+
+	public void findByFingerprints(final int maxCount, Collection<String> fingerprints, final BookSearchCallback callback, final Handler handler) {
+		execTask(new Task("findByFingerprint") {
+			@Override
+			public void work() {
+				final ArrayList<FileInfo> list = mainDB.findByFingerprints(maxCount, fingerprints);
+				sendTask(handler, () -> callback.onBooksFound(list));
 			}
 		});
 	}
@@ -507,27 +484,17 @@ public class CRDBService extends Service {
 			@Override
 			public void work() {
 				final BookInfo bookInfo = mainDB.loadBookInfo(fileInfo);
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onBooksInfoLoaded(bookInfo);
-					}
-				});
+				sendTask(handler, () -> callback.onBooksInfoLoaded(bookInfo));
 			}
 		});
 	}
 
-	public void loadFileInfos(final ArrayList<String> pathNames, final FileInfoLoadingCallback callback, final Handler handler) {
+	public void loadFileInfos(final ArrayList<String> pathNames, final Scanner.ScanControl control, final Engine.ProgressControl progress, final FileInfoLoadingCallback callback, final Handler handler) {
 		execTask(new Task("loadFileInfos") {
 			@Override
 			public void work() {
-				final ArrayList<FileInfo> list = mainDB.loadFileInfos(pathNames);
-				sendTask(handler, new Runnable() {
-					@Override
-					public void run() {
-						callback.onFileInfoListLoaded(list);
-					}
-				});
+				final ArrayList<FileInfo> list = mainDB.loadFileInfos(pathNames, control, progress);
+				sendTask(handler, () -> callback.onFileInfoListLoaded(list));
 			}
 		});
 	}
@@ -600,12 +567,7 @@ public class CRDBService extends Service {
             @Override
             public void work() {
                 final ArrayList<FileInfo> favorites = mainDB.loadFavoriteFolders();
-                sendTask(handler, new Runnable() {
-                    @Override
-                    public void run() {
-                        callback.onFileInfoListLoaded(favorites);
-                    }
-                });
+                sendTask(handler, () -> callback.onFileInfoListLoaded(favorites));
             }
         });
    	}
@@ -630,72 +592,6 @@ public class CRDBService extends Service {
         flush();
    	}
 
-	private abstract class Task implements Runnable {
-		private final String name;
-		public Task(String name) {
-			this.name = name;
-		}
-
-		@Override
-		public String toString() {
-			return "Task[" + name + "]";
-		}
-
-		@Override
-		public void run() {
-			long ts = Utils.timeStamp();
-			vlog.v(toString() + " started");
-			try {
-				work();
-			} catch (Exception e) {
-				log.e("Exception while running DB task in background", e);
-			}
-			vlog.v(toString() + " finished in " + Utils.timeInterval(ts) + " ms");
-		}
-		
-		public abstract void work();
-	}
-	
-	/**
-	 * Execute runnable in CDRDBService background thread.
-	 * Exceptions will be ignored, just dumped into log.
-	 * @param task is Runnable to execute
-	 */
-	private void execTask(final Task task) {
-		vlog.v("Posting task " + task);
-		mThread.post(task);
-	}
-	
-	/**
-	 * Execute runnable in CDRDBService background thread, delayed.
-	 * Exceptions will be ignored, just dumped into log.
-	 * @param task is Runnable to execute
-	 */
-	private void execTask(final Task task, long delay) {
-		vlog.v("Posting task " + task + " with delay " + delay);
-		mThread.postDelayed(task, delay);
-	}
-	
-	/**
-	 * Send task to handler, if specified, otherwise run immediately.
-	 * Exceptions will be ignored, just dumped into log.
-	 * @param handler is handler to send task to, null to run immediately
-	 * @param task is Runnable to execute
-	 */
-	private void sendTask(Handler handler, Runnable task) {
-		try {
-			if (handler != null) {
-				vlog.v("Senging task to " + handler.toString());
-				handler.post(task);
-			} else {
-				vlog.v("No Handler provided: executing task in current thread");
-				task.run();
-			}
-		} catch (Exception e) {
-			log.e("Exception in DB callback", e);
-		}
-	}
-
 	/**
      * Class for clients to access.  Because we know this service always
      * runs in the same process as its clients, we don't need to deal with
@@ -731,7 +627,11 @@ public class CRDBService extends Service {
     		getService().removeOPDSCatalog(id);
     	}
 
-    	public void loadAuthorsList(FileInfo parent, final ItemGroupsLoadingCallback callback) {
+		public void loadGenresList(FileInfo parent, boolean showEmptyGenres, final ItemGroupsLoadingCallback callback) {
+			getService().loadGenresList(parent, showEmptyGenres, callback, new Handler());
+		}
+
+		public void loadAuthorsList(FileInfo parent, final ItemGroupsLoadingCallback callback) {
     		getService().loadAuthorsList(parent, callback, new Handler());
     	}
 
@@ -743,6 +643,10 @@ public class CRDBService extends Service {
     		getService().loadTitleList(parent, callback, new Handler());
     	}
 
+		public void loadGenresBooks(String genreCode, boolean showEmptyGenres, FileInfoLoadingCallback callback) {
+			getService().findGenresBooks(genreCode, showEmptyGenres, callback, new Handler());
+		}
+
     	public void loadAuthorBooks(long authorId, FileInfoLoadingCallback callback) {
     		getService().findAuthorBooks(authorId, callback, new Handler());
     	}
@@ -774,13 +678,17 @@ public class CRDBService extends Service {
     	public void saveFileInfos(final Collection<FileInfo> list) {
     		getService().saveFileInfos(deepCopyFileInfos(list));
     	}
-    	
-    	public void findByPatterns(final int maxCount, final String author, final String title, final String series, final String filename, final BookSearchCallback callback) {
-    		getService().findByPatterns(maxCount, author, title, series, filename, callback, new Handler());
+
+    	public void findByFingerprints(final int maxCount, Collection<String> fingerprints, final BookSearchCallback callback) {
+    		getService().findByFingerprints(maxCount, fingerprints, callback, new Handler());
     	}
-    	
-    	public void loadFileInfos(final ArrayList<String> pathNames, final FileInfoLoadingCallback callback) {
-    		getService().loadFileInfos(pathNames, callback, new Handler());
+
+		public void findByPatterns(final int maxCount, final String authors, final String title, final String series, final String filename, final BookSearchCallback callback) {
+			getService().findByPatterns(maxCount, authors, title, series, filename, callback, new Handler());
+		}
+
+		public void loadFileInfos(final ArrayList<String> pathNames, final Scanner.ScanControl control, final Engine.ProgressControl progress, final FileInfoLoadingCallback callback) {
+    		getService().loadFileInfos(pathNames, control, progress, callback, new Handler());
     	}
 
     	public void deleteBook(final FileInfo fileInfo)	{
@@ -854,7 +762,6 @@ public class CRDBService extends Service {
         return true;
     }
 
-    private ServiceThread mThread;
     private final IBinder mBinder = new LocalBinder();
     
 }
