@@ -9,6 +9,7 @@
 #include <qglobal.h>
 #if QT_VERSION >= 0x050000
 #include <QResizeEvent>
+#include <QtGui/QScreen>
 #include <QtWidgets/QScrollBar>
 #include <QtWidgets/QMenu>
 #include <QtWidgets/QStyleFactory>
@@ -28,17 +29,20 @@
 #include <QDir>
 #include <QFileInfo>
 #include <QDesktopServices>
+#include <QLocale>
 
 /// to hide non-qt implementation, place all crengine-related fields here
 class CR3View::DocViewData
 {
     friend class CR3View;
-    lString16 _settingsFileName;
-    lString16 _historyFileName;
+    lString32 _settingsFileName;
+    lString32 _historyFileName;
     CRPropRef _props;
 };
 
+#if USE_LIMITED_FONT_SIZES_SET
 DECL_DEF_CR_FONT_SIZES;
+#endif
 
 static void replaceColor( char * str, lUInt32 color )
 {
@@ -84,8 +88,6 @@ static LVRefVec<LVImageSource>& getBatteryIcons( lUInt32 color )
         ".0..0.XXXX.XXXX.XXXX.XXXX.0.",
         ".0..0.XXXX.XXXX.XXXX.XXXX.0.",
         ".0..0.XXXX.XXXX.XXXX.XXXX.0.",
-        ".0..0.XXXX.XXXX.XXXX.XXXX.0.",
-        ".0..0.XXXX.XXXX.XXXX.XXXX.0.",
         ".0000.XXXX.XXXX.XXXX.XXXX.0.",
         "....0.XXXX.XXXX.XXXX.XXXX.0.",
         "   .0.....................0.",
@@ -307,7 +309,10 @@ CR3View::CR3View( QWidget *parent)
         : QWidget( parent, Qt::WindowFlags() ), _scroll(NULL), _propsCallback(NULL)
         , _normalCursor(Qt::ArrowCursor), _linkCursor(Qt::PointingHandCursor)
         , _selCursor(Qt::IBeamCursor), _waitCursor(Qt::WaitCursor)
-        , _selecting(false), _selected(false), _editMode(false), _lastBatteryState(CR_BATTERY_STATE_NO_BATTERY)
+        , _selecting(false), _selected(false), _editMode(false)
+        , _lastBatteryState(CR_BATTERY_STATE_NO_BATTERY)
+        , _lastBatteryChargingConn(CR_BATTERY_CHARGER_NO)
+        , _lastBatteryChargeLevel(0)
 {
 #if WORD_SELECTOR_ENABLED==1
     _wordSelector = NULL;
@@ -323,11 +328,13 @@ CR3View::CR3View( QWidget *parent)
     ldomXPointerEx p2;
     _selRange.setStart(p1);
     _selRange.setEnd(p2);
+#if USE_LIMITED_FONT_SIZES_SET
     LVArray<int> sizes( cr_font_sizes, sizeof(cr_font_sizes)/sizeof(int) );
     _docview->setFontSizes( sizes, false );
+#endif
 
     _docview->setBatteryIcons( getBatteryIcons(0x000000) );
-    _docview->setBatteryState(CR_BATTERY_STATE_NO_BATTERY); // don't show battery
+    _docview->setBatteryState(CR_BATTERY_STATE_NO_BATTERY, CR_BATTERY_CHARGER_NO, 0); // don't show battery
     //_docview->setBatteryState( 75 ); // 75%
 //    LVStreamRef stream;
 //    stream = LVOpenFileStream("/home/lve/.cr3/textures/old_paper.png", LVOM_READ);
@@ -343,6 +350,7 @@ CR3View::CR3View( QWidget *parent)
     updateDefProps();
     setMouseTracking(true);
     setFocusPolicy(Qt::StrongFocus);
+    setMinimumSize(SCREEN_SIZE_MIN, SCREEN_SIZE_MIN);
 }
 
 void CR3View::updateDefProps()
@@ -354,6 +362,14 @@ void CR3View::updateDefProps()
     _data->_props->setStringDef( PROP_WINDOW_SHOW_STATUSBAR, "0" );
     _data->_props->setStringDef( PROP_APP_START_ACTION, "0" );
 
+    if (QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)) {
+        QScreen* screen = QGuiApplication::primaryScreen();
+        lString32 str;
+        str.appendDecimal((int)screen->logicalDotsPerInch());
+        _data->_props->setString( PROP_RENDER_DPI, str );
+        // But we don't apply PROP_RENDER_SCALE_FONT_WITH_DPI property,
+        // since for now we are setting the font size in pixels.
+    }
 
     QStringList styles = QStyleFactory::keys();
     QStyle * s = QApplication::style();
@@ -444,7 +460,7 @@ void CR3View::setDocumentText( QString text )
 {
     _docview->savePosition();
     clearSelection();
-    _docview->createDefaultDocument(lString16::empty_str, qt2cr(text));
+    _docview->createDefaultDocument(lString32::empty_str, qt2cr(text));
 }
 
 bool CR3View::loadLastDocument()
@@ -461,13 +477,19 @@ bool CR3View::loadDocument( QString fileName )
     clearSelection();
     bool res = _docview->LoadDocument( qt2cr(fileName).c_str() );
     if ( res ) {
+        CRPropRef props = _docview->propsGetCurrent();
+        if (props->getBoolDef(PROP_TEXTLANG_EMBEDDED_LANGS_ENABLED, false)) {
+            lString32 doc_lang = _docview->getLanguage();
+            if (!doc_lang.empty())
+                _docview->propApply(lString8(PROP_TEXTLANG_MAIN_LANG), doc_lang);
+        }
         //_docview->swapToCache();
         QByteArray utf8 = fileName.toUtf8();
         CRLog::debug( "Trying to restore position for %s", utf8.constData() );
         _docview->restorePosition();
         checkFontLanguageCompatibility();
     } else {
-        _docview->createDefaultDocument(lString16::empty_str, qt2cr(tr("Error while opening document ") + fileName));
+        _docview->createDefaultDocument(lString32::empty_str, qt2cr(tr("Error while opening document ") + fileName));
     }
     update();
     return res;
@@ -475,7 +497,12 @@ bool CR3View::loadDocument( QString fileName )
 
 void CR3View::wheelEvent( QWheelEvent * event )
 {
+    // Get degrees delta from vertical scrolling
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+    int numDegrees = event->angleDelta().y() / 8;
+#else
     int numDegrees = event->delta() / 8;
+#endif
     int numSteps = numDegrees / 15;
     if ( numSteps==0 && numDegrees!=0 )
         numSteps = numDegrees>0 ? 1 : -1;
@@ -503,21 +530,31 @@ void CR3View::resizeEvent ( QResizeEvent * event )
     _docview->Resize( sz.width(), sz.height() );
 }
 
-int getBatteryState()
+static bool getBatteryState(int& state, int& chargingConn, int& level)
 {
 #ifdef _WIN32
     // update battery state
     SYSTEM_POWER_STATUS bstatus;
     BOOL pow = GetSystemPowerStatus(&bstatus);
-    if (bstatus.BatteryFlag & 128)
-        return CR_BATTERY_STATE_NO_BATTERY; // no system battery
-	if (bstatus.ACLineStatus==8 && bstatus.BatteryLifePercent<100 )
-		return CR_BATTERY_STATE_CHARGING; // charging
-    if (bstatus.BatteryLifePercent>=0 && bstatus.BatteryLifePercent<=100)
-		return bstatus.BatteryLifePercent;
-    return CR_BATTERY_STATE_NO_BATTERY;
+    if (pow) {
+        state = CR_BATTERY_STATE_DISCHARGING;
+        if (bstatus.BatteryFlag & 128)
+            state = CR_BATTERY_STATE_NO_BATTERY;  // no system battery
+        else if (bstatus.BatteryFlag & 8)
+            state = CR_BATTERY_STATE_CHARGING;    // charging
+        chargingConn = CR_BATTERY_CHARGER_NO;
+        if (bstatus.ACLineStatus==1)
+            chargingConn = CR_BATTERY_CHARGER_AC; // AC power charging connected
+        if (bstatus.BatteryLifePercent>=0 && bstatus.BatteryLifePercent<=100)
+            level = bstatus.BatteryLifePercent;
+        return true;
+    }
+    return false;
 #else
-	return CR_BATTERY_STATE_NO_BATTERY;
+    state = CR_BATTERY_STATE_NO_BATTERY;
+    chargingConn = CR_BATTERY_CHARGER_NO;
+    level = 0;
+    return true;
 #endif
 }
 
@@ -525,11 +562,18 @@ void CR3View::paintEvent ( QPaintEvent * event )
 {
     QPainter painter(this);
     QRect rc = rect();
-	int newBatteryState = getBatteryState();
-	if (_lastBatteryState != newBatteryState) {
-		_docview->setBatteryState( newBatteryState );
-		_lastBatteryState = newBatteryState;
-	}
+    int newBatteryState;
+    int newChargingConn;
+    int newChargeLevel;
+    if (getBatteryState(newBatteryState, newChargingConn, newChargeLevel) &&
+            (_lastBatteryState != newBatteryState ||
+             _lastBatteryChargingConn != newChargingConn ||
+             _lastBatteryChargeLevel != newChargeLevel)) {
+        _docview->setBatteryState( newBatteryState, newChargingConn, newChargeLevel );
+        _lastBatteryState = newBatteryState;
+        _lastBatteryChargingConn = newChargingConn;
+        _lastBatteryChargeLevel = newChargeLevel;
+    }
     LVDocImageRef ref = _docview->getPageImage(0);
     if ( ref.isNull() ) {
         //painter.fillRect();
@@ -749,6 +793,11 @@ void CR3View::refreshPropFromView( const char * propName )
     _data->_props->setString( propName, _docview->propsGetCurrent()->getStringDef( propName, "" ) );
 }
 
+QSize CR3View::minimumSizeHint() const
+{
+    return QSize(SCREEN_SIZE_MIN, SCREEN_SIZE_MIN);
+}
+
 void CR3View::zoomIn()
 { 
     doCommand( DCMD_ZOOM_IN, 1 );
@@ -778,7 +827,7 @@ void CR3View::setScrollBar( QScrollBar * scroll )
 /// load fb2.css file
 bool CR3View::loadCSS( QString fn )
 {
-    lString16 filename( qt2cr(fn) );
+    lString32 filename( qt2cr(fn) );
     lString8 css;
     if ( LVLoadStylesheetFile( filename, css ) ) {
         if ( !css.empty() ) {
@@ -795,12 +844,12 @@ bool CR3View::loadCSS( QString fn )
 /// load settings from file
 bool CR3View::loadSettings( QString fn )
 {
-    lString16 filename( qt2cr(fn) );
+    lString32 filename( qt2cr(fn) );
     _data->_settingsFileName = filename;
     LVStreamRef stream = LVOpenFileStream( filename.c_str(), LVOM_READ );
     bool res = false;
     if ( !stream.isNull() && _data->_props->loadFromStream( stream.get() ) ) {
-        CRLog::error("Loading settings from file %s", fn.toUtf8().data() );
+        CRLog::info("Loading settings from file %s", fn.toUtf8().data() );
         res = true;
     } else {
         CRLog::error("Cannot load settings from file %s", fn.toUtf8().data() );
@@ -869,7 +918,7 @@ PropsRef CR3View::getOptions()
 /// save settings from file
 bool CR3View::saveSettings( QString fn )
 {
-    lString16 filename( qt2cr(fn) );
+    lString32 filename( qt2cr(fn) );
     crtrace log;
     if ( filename.empty() )
         filename = _data->_settingsFileName;
@@ -879,7 +928,7 @@ bool CR3View::saveSettings( QString fn )
     log << "V3DocViewWin::saveSettings(" << filename << ")";
     LVStreamRef stream = LVOpenFileStream( filename.c_str(), LVOM_WRITE );
     if ( !stream ) {
-        lString16 path16 = LVExtractPath( filename );
+        lString32 path16 = LVExtractPath( filename );
         lString8 path = UnicodeToUtf8(path16);
         if ( !LVCreateDirectory( path16 ) ) {
             CRLog::error("Cannot create directory %s", path.c_str() );
@@ -898,7 +947,7 @@ bool CR3View::saveSettings( QString fn )
 /// load history from file
 bool CR3View::loadHistory( QString fn )
 {
-    lString16 filename( qt2cr(fn) );
+    lString32 filename( qt2cr(fn) );
     CRLog::trace("V3DocViewWin::loadHistory( %s )", UnicodeToUtf8(filename).c_str());
     _data->_historyFileName = filename;
     LVStreamRef stream = LVOpenFileStream( filename.c_str(), LVOM_READ );
@@ -913,7 +962,7 @@ bool CR3View::loadHistory( QString fn )
 /// save history to file
 bool CR3View::saveHistory( QString fn )
 {
-    lString16 filename( qt2cr(fn) );
+    lString32 filename( qt2cr(fn) );
     crtrace log;
     if ( filename.empty() )
         filename = _data->_historyFileName;
@@ -923,14 +972,14 @@ bool CR3View::saveHistory( QString fn )
     }
     //CRLog::debug("Exporting bookmarks to %s", UnicodeToUtf8(_bookmarkDir).c_str());
     //_docview->exportBookmarks(_bookmarkDir); //use default filename
-    lString16 bmdir = qt2cr(_bookmarkDir);
+    lString32 bmdir = qt2cr(_bookmarkDir);
     LVAppendPathDelimiter( bmdir );
     _docview->exportBookmarks( bmdir ); //use default filename
     _data->_historyFileName = filename;
     log << "V3DocViewWin::saveHistory(" << filename << ")";
     LVStreamRef stream = LVOpenFileStream( filename.c_str(), LVOM_WRITE );
     if ( !stream ) {
-        lString16 path16 = LVExtractPath( filename );
+        lString32 path16 = LVExtractPath( filename );
         lString8 path = UnicodeToUtf8(path16);
         if ( !LVCreateDirectory( path16 ) ) {
             CRLog::error("Cannot create directory %s", path.c_str() );
@@ -968,8 +1017,8 @@ void CR3View::mouseMoveEvent ( QMouseEvent * event )
     //bool mid = (event->buttons() & Qt::MidButton);
     lvPoint pt ( event->x(), event->y() );
     ldomXPointer p = _docview->getNodeByPoint( pt );
-    lString16 path;
-    lString16 href;
+    lString32 path;
+    lString32 href;
     if ( !p.isNull() ) {
         path = p.toString();
         href = p.getHRef();
@@ -1043,7 +1092,7 @@ bool CR3View::updateSelection( ldomXPointer p )
             r.getStart().prevVisibleWordStart();
             //CRLog::trace("updated : %s", LCSTR(r.getStart().toString()));
         }
-        //lString16 start = r.getStart().toString();
+        //lString32 start = r.getStart().toString();
         if ( !r.getEnd().isVisibleWordEnd() ) {
             //CRLog::trace("calling nextVisibleWordEnd : %s", LCSTR(r.getEnd().toString()));
             r.getEnd().nextVisibleWordEnd();
@@ -1052,7 +1101,7 @@ bool CR3View::updateSelection( ldomXPointer p )
     }
     if ( r.isNull() )
         return false;
-    //lString16 end = r.getEnd().toString();
+    //lString32 end = r.getEnd().toString();
     //CRLog::debug("Range: %s - %s", UnicodeToUtf8(start).c_str(), UnicodeToUtf8(end).c_str());
     r.setFlags(1);
     _docview->selectRange( r );
@@ -1065,25 +1114,42 @@ bool CR3View::updateSelection( ldomXPointer p )
 
 void CR3View::checkFontLanguageCompatibility()
 {
-    lString16 fontFace;
+#if USE_LOCALE_DATA==1
+    lString32 fontFace;
+    lString32 fileName;
     _data->_props->getString(PROP_FONT_FACE, fontFace);
+    _docview->getDocProps()->getString(DOC_PROP_FILE_NAME, fileName);
     lString8 fontFace_u8 = UnicodeToUtf8(fontFace);
-    lString16 langCode = _docview->getLanguage();
+    lString32 langCode = _docview->getLanguage();
     lString8 langCode_u8 = UnicodeToUtf8(langCode);
     if (langCode_u8.length() == 0) {
         CRLog::debug("Can't fetch book's language to check font compatibility!");
         return;
     }
     if (fontFace_u8.length() > 0) {
-        bool res = fontMan->checkFontLangCompat(fontFace_u8, langCode_u8);
-        CRLog::debug("Checking font \"%s\" for compatibility with language \"%s\": %d", fontFace_u8.c_str(), langCode_u8.c_str(), res);
-        if (!res)
-        {
-            QMessageBox::warning(this, tr("Warning"), 
-                                        tr("Font \"%1\" isn't compatible with language \"%2\". Instead will be used fallback font.").arg(fontFace_u8.c_str()).arg(langCode_u8.c_str()), 
-                                        QMessageBox::Ok);
+        QString langDescr = getHumanReadableLocaleName(langCode);
+        if (!langDescr.isEmpty()) {
+            font_lang_compat compat = fontMan->checkFontLangCompat(fontFace_u8, langCode_u8);
+            CRLog::debug("Checking font \"%s\" for compatibility with language \"%s\": %d", fontFace_u8.c_str(), langCode_u8.c_str(), (int)compat);
+            switch (compat) {
+                case font_lang_compat_invalid_tag:
+                    CRLog::warn("Can't find compatible language code in embedded FontConfig catalog: language=\"%s\", filename=\"%s\"", langCode_u8.c_str(), LCSTR(fileName));
+                    break;
+                case font_lang_compat_none:
+                case font_lang_compat_partial:
+                    QMessageBox::warning(this, tr("Warning"), 
+                                         tr("Font \"%1\" isn't compatible with language \"%2\". Instead will be used fallback font.").arg(fontFace_u8.c_str()).arg(langDescr), 
+                                         QMessageBox::Ok);
+                    break;
+                case font_lang_compat_full:
+                    // good, do nothing
+                    break;
+            }
+        } else {
+            CRLog::warn("Invalid language tag: \"%s\", filename=\"%s\"", langCode_u8.c_str(), LCSTR(fileName));
         }
     }
+#endif
 }
 
 void CR3View::mousePressEvent ( QMouseEvent * event )
@@ -1103,8 +1169,8 @@ void CR3View::mousePressEvent ( QMouseEvent * event )
         if (mid)
             AddBookmarkDialog::editBookmark((QWidget*)parent(), this, bmk);
     }
-    lString16 path;
-    lString16 href;
+    lString32 path;
+    lString32 href;
     if ( !p.isNull() ) {
         path = p.toString();
         CRLog::debug("mousePressEvent(%s)", LCSTR(path));
@@ -1137,8 +1203,8 @@ void CR3View::mouseReleaseEvent ( QMouseEvent * event )
     //bool mid = event->button() == Qt::MidButton;
     lvPoint pt (event->x(), event->y());
     ldomXPointer p = _docview->getNodeByPoint( pt );
-    lString16 path;
-    lString16 href;
+    lString32 path;
+    lString32 href;
     if ( !p.isNull() ) {
         path = p.toString();
         href = p.getHRef();
@@ -1164,7 +1230,7 @@ void CR3View::mouseReleaseEvent ( QMouseEvent * event )
 }
 
 /// Override to handle external links
-void CR3View::OnExternalLink( lString16 url, ldomNode * node )
+void CR3View::OnExternalLink( lString32 url, ldomNode * node )
 {
     // TODO: add support of file links
     // only URL supported for now
@@ -1177,9 +1243,9 @@ CRBookmark * CR3View::createBookmark()
 {
     CRBookmark * bm = NULL;
     if ( getSelectionText().length()>0 && !_selRange.isNull() ) {
-        bm = getDocView()->saveRangeBookmark(_selRange, bmkt_comment, lString16::empty_str);
+        bm = getDocView()->saveRangeBookmark(_selRange, bmkt_comment, lString32::empty_str);
     } else {
-        bm = getDocView()->saveCurrentPageBookmark(lString16::empty_str);
+        bm = getDocView()->saveCurrentPageBookmark(lString32::empty_str);
     }
 
     return bm;
@@ -1226,6 +1292,9 @@ void CR3View::OnLoadFileFormatDetected( doc_format_t fileFormat )
         case doc_format_txt:
             filename = "txt.css";
             break;
+        case doc_format_fb3:
+            filename = "fb3.css";
+            break;
         case doc_format_rtf:
             filename = "rtf.css";
             break;
@@ -1238,6 +1307,10 @@ void CR3View::OnLoadFileFormatDetected( doc_format_t fileFormat )
         case doc_format_doc:
             filename = "doc.css";
             break;
+        case doc_format_odt:
+        case doc_format_docx:
+            filename = "docx.css";
+            break;
         case doc_format_chm:
             filename = "chm.css";
             break;
@@ -1255,13 +1328,13 @@ void CR3View::OnLoadFileFormatDetected( doc_format_t fileFormat )
 }
 
 /// on starting file loading
-void CR3View::OnLoadFileStart( lString16 filename )
+void CR3View::OnLoadFileStart( lString32 filename )
 {
     setCursor( _waitCursor );
 }
 
 /// file load finiished with error
-void CR3View::OnLoadFileError( lString16 message )
+void CR3View::OnLoadFileError( lString32 message )
 {
     setCursor( _normalCursor );
 }
@@ -1411,7 +1484,7 @@ void CR3View::OnLoadFileFirstPagesReady()
     //update();
     repaint();
     CRLog::info( "OnLoadFileFirstPagesReady() - painting done" );
-    _docview->setPageHeaderOverride(lString16::empty_str);
+    _docview->setPageHeaderOverride(lString32::empty_str);
     _docview->requestRender();
     // TODO: remove debug sleep
     //sleep(5);
