@@ -13,19 +13,19 @@ class CR3View;
 class SearchDialog : public QDialog {
     Q_OBJECT
 public:
+    static bool showDlg( QWidget * parent, CR3View * docView );
+    bool findText( lString32 pattern, int origin, bool reverse, bool caseInsensitive );
+protected:
     SearchDialog(QWidget *parent, CR3View * docView);
     ~SearchDialog();
-
-    static bool showDlg( QWidget * parent, CR3View * docView );
-
-    bool findText( lString16 pattern, int origin, bool reverse, bool caseInsensitive );
-protected:
     void changeEvent(QEvent *e);
+    void closeEvent(QCloseEvent* e);
 
 private:
     Ui::SearchDialog *ui;
     CR3View * _docview;
-    lString16 _lastPattern;
+    lString32 _lastPattern;
+    static SearchDialog* _instance;
 private slots:
     void on_btnFindNext_clicked();
     void on_btnClose_clicked();
