@@ -2,6 +2,7 @@
 #include "ui_settings.h"
 #include "cr3widget.h"
 #include "crqtutil.h"
+#include "fallbackfontsdialog.h"
 #include <qglobal.h>
 #if QT_VERSION >= 0x050000
 #include <QtWidgets/QColorDialog>
@@ -12,14 +13,31 @@
 #endif
 #include <QDir>
 
+#include "lvrend.h"         // for BLOCK_RENDERING_FLAGS_XXX presets
+
 static int def_margins[] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, 20, 25, 30, 40, 50, 60 };
 #define MAX_MARGIN_INDEX (sizeof(def_margins)/sizeof(int))
 
 DECL_DEF_CR_FONT_SIZES;
 
-static bool initDone = false;
+#if USE_FT_EMBOLDEN
+static int synth_weights[] = { 300, 400, 500, 600, 700, 800, 900 };
+#else
+static int synth_weights[] = { 600 };
+#endif
+#define SYNTH_WEIGHTS_SZ    (sizeof(synth_weights)/sizeof(int))
 
-static void findImagesFromDirectory( lString16 dir, lString16Collection & files ) {
+static int rend_flags[] = { BLOCK_RENDERING_FLAGS_LEGACY, BLOCK_RENDERING_FLAGS_FLAT, BLOCK_RENDERING_FLAGS_BOOK, BLOCK_RENDERING_FLAGS_WEB };
+#define MAX_REND_FLAGS_INDEX (sizeof(rend_flags)/sizeof(int))
+static int DOM_versions[] = { 0, gDOMVersionCurrent };
+#define MAX_DOM_VERSIONS_INDEX (sizeof(DOM_versions)/sizeof(int))
+static int aa_variants[] = { font_aa_none, font_aa_gray, font_aa_lcd_rgb, font_aa_lcd_bgr, font_aa_lcd_v_rgb, font_aa_lcd_v_bgr };
+#define MAX_AA_INDEX (sizeof(aa_variants)/sizeof(int))
+
+static bool initDone = false;
+static bool suppressOnChange = false;
+
+static void findImagesFromDirectory( lString32 dir, lString32Collection & files ) {
     LVAppendPathDelimiter(dir);
     if ( !LVDirectoryExists(dir) )
         return;
@@ -28,7 +46,7 @@ static void findImagesFromDirectory( lString16 dir, lString16Collection & files
         for ( int i=0; i<cont->GetObjectCount(); i++ ) {
             const LVContainerItemInfo * item  = cont->GetObjectInfo(i);
             if ( !item->IsContainer() ) {
-                lString16 name = item->GetName();
+                lString32 name = item->GetName();
                 name.lowercase();
                 if ( name.endsWith(".png") || name.endsWith(".jpg") || name.endsWith(".gif")
                     || name.endsWith(".jpeg") ) {
@@ -39,20 +57,60 @@ static void findImagesFromDirectory( lString16 dir, lString16Collection & files
     }
 }
 
-static void findBackgrounds( lString16Collection & baseDirs, lString16Collection & files ) {
+static void findBackgrounds( lString32Collection & baseDirs, lString32Collection & files ) {
     int i;
     for ( i=0; i<baseDirs.length(); i++ ) {
-        lString16 baseDir = baseDirs[i];
+        lString32 baseDir = baseDirs[i];
         LVAppendPathDelimiter(baseDir);
         findImagesFromDirectory( baseDir + "backgrounds", files );
     }
     for ( i=0; i<baseDirs.length(); i++ ) {
-        lString16 baseDir = baseDirs[i];
+        lString32 baseDir = baseDirs[i];
         LVAppendPathDelimiter(baseDir);
         findImagesFromDirectory( baseDir + "textures", files );
     }
 }
 
+static QString getWeightName(int weight) {
+    QString name;
+    switch (weight) {
+    case 100:
+        name = QString("Thin");
+        break;
+    case 200:
+        name = QString("ExtraLight");
+        break;
+    case 300:
+        name = QString("Light");
+        break;
+    case 350:
+        name = QString("Book");
+        break;
+    case 400:
+        name = QString("Regular");
+        break;
+    case 500:
+        name = QString("Medium");
+        break;
+    case 600:
+        name = QString("SemiBold");
+        break;
+    case 700:
+        name = QString("Bold");
+        break;
+    case 800:
+        name = QString("ExtraBold");
+        break;
+    case 900:
+        name = QString("Heavy/Black");
+        break;
+    case 950:
+        name = QString("ExtraBlack");
+        break;
+    }
+    return name;
+}
+
 static const char * styleNames[] = {
     "def",
     "title",
@@ -97,13 +155,13 @@ SettingsDlg::SettingsDlg(QWidget *parent, CR3View * docView ) :
     QString exeDir = QDir::toNativeSeparators(qApp->applicationDirPath() + "/"); //QDir::separator();
 #endif
 
-    lString16Collection baseDirs;
+    lString32Collection baseDirs;
     baseDirs.add(qt2cr(homeDir));
     baseDirs.add(qt2cr(exeDir));
 #ifdef _LINUX
-    baseDirs.add(cs16("/usr/local/share/cr3/"));
+    baseDirs.add(cs32("/usr/local/share/cr3/"));
 #endif
-    lString16Collection bgFiles;
+    lString32Collection bgFiles;
     QStringList bgFileLabels;
     findBackgrounds( baseDirs, bgFiles );
     int bgIndex = 0;
@@ -111,7 +169,7 @@ SettingsDlg::SettingsDlg(QWidget *parent, CR3View * docView ) :
     bgFileLabels.append("[NONE]");
     QString bgFile = m_props->getStringDef(PROP_BACKGROUND_IMAGE, "");
     for ( int i=0; i<bgFiles.length(); i++ ) {
-        lString16 fn = bgFiles[i];
+        lString32 fn = bgFiles[i];
         QString f = cr2qt(fn);
         if ( f==bgFile )
             bgIndex = i + 1;
@@ -137,13 +195,7 @@ SettingsDlg::SettingsDlg(QWidget *parent, CR3View * docView ) :
     optionToUi( PROP_EMBEDDED_STYLES, m_ui->cbEnableDocumentStyles );
     optionToUi( PROP_EMBEDDED_FONTS, m_ui->cbEnableEmbeddedFonts );
     m_ui->cbEnableEmbeddedFonts->setEnabled(m_props->getBoolDef(PROP_EMBEDDED_STYLES, true));
-    optionToUi( PROP_TXT_OPTION_PREFORMATTED, m_ui->cbTxtPreFormatted );
     optionToUi( PROP_FONT_KERNING_ENABLED, m_ui->cbFontKerning );
-#if USE_HARFBUZZ==1
-    optionToUi( PROP_FONT_LIGATURES_ENABLED, m_ui->cbLigatures );
-#else
-    m_ui->cbLigatures->setEnabled(false);
-#endif
     optionToUi( PROP_FLOATING_PUNCTUATION, m_ui->cbFloatingPunctuation );
     optionToUiIndex( PROP_IMG_SCALING_ZOOMIN_INLINE_MODE, m_ui->cbImageInlineZoominMode );
     optionToUiIndex( PROP_IMG_SCALING_ZOOMIN_INLINE_SCALE, m_ui->cbImageInlineZoominScale );
@@ -159,9 +211,12 @@ SettingsDlg::SettingsDlg(QWidget *parent, CR3View * docView ) :
         m_props->setString(PROP_FONT_GAMMA, "1.0");
     optionToUiString(PROP_FONT_GAMMA, m_ui->cbFontGamma);
 
-    optionToUiInversed( PROP_STATUS_LINE, m_ui->cbShowPageHeader );
+    QString ignoreDocMargins = m_props->getStringDef("styles.body.margin", "");
+    m_ui->cbIgnoreDocumentMargins->setCheckState(ignoreDocMargins.length() > 0 ?  Qt::Checked : Qt::Unchecked);
 
-    bool b = m_props->getIntDef( PROP_STATUS_LINE, 0 )==0;
+    optionToUiIndex( PROP_STATUS_LINE, m_ui->cbShowPageHeader );
+
+    bool b = m_props->getIntDef( PROP_STATUS_LINE, 0 ) > 0;
     m_ui->cbShowBattery->setEnabled( b );
     m_ui->cbShowClock->setEnabled( b );
     m_ui->cbShowBookName->setEnabled( b );
@@ -178,9 +233,30 @@ SettingsDlg::SettingsDlg(QWidget *parent, CR3View * docView ) :
         m_ui->cbViewMode->setCurrentIndex( lp==1 ? 0 : 1 );
     int hinting = m_props->getIntDef(PROP_FONT_HINTING, 2);
     m_ui->cbFontHinting->setCurrentIndex(hinting);
+    int shaping = m_props->getIntDef(PROP_FONT_SHAPING, 1);
+    m_ui->cbFontShaping->setCurrentIndex(shaping);
     int highlight = m_props->getIntDef(PROP_HIGHLIGHT_COMMENT_BOOKMARKS, 1);
     m_ui->cbBookmarkHighlightMode->setCurrentIndex(highlight);
 
+    int rendFlagsIndex = 0;
+    int rendFlags = m_props->getIntDef(PROP_RENDER_BLOCK_RENDERING_FLAGS, BLOCK_RENDERING_FLAGS_WEB);
+    for (int i = 0; i < MAX_REND_FLAGS_INDEX; i++) {
+        if (rendFlags == rend_flags[i]) {
+            rendFlagsIndex = i;
+            break;
+        }
+    }
+    m_ui->cbRendFlags->setCurrentIndex(rendFlagsIndex);
+
+    int DOMVersionIndex = 0;
+    int DOMVersion = m_props->getIntDef(PROP_REQUESTED_DOM_VERSION, gDOMVersionCurrent);
+    for (int i = 0; i < MAX_DOM_VERSIONS_INDEX; i++) {
+        if (DOMVersion == DOM_versions[i]) {
+            DOMVersionIndex = i;
+            break;
+        }
+    }
+    m_ui->cbDOMLevel->setCurrentIndex(DOMVersionIndex);
 
     int n = m_props->getIntDef( PROP_PAGE_MARGIN_LEFT, 8 );
     int mi = 0;
@@ -208,7 +284,6 @@ SettingsDlg::SettingsDlg(QWidget *parent, CR3View * docView ) :
     crGetFontFaceList( m_faceList );
     m_ui->cbTextFontFace->addItems( m_faceList );
     m_ui->cbTitleFontFace->addItems( m_faceList );
-    m_ui->cbFallbackFontFace->addItems( m_faceList );
     QStringList sizeList;
     LVArray<int> sizes( cr_font_sizes, sizeof(cr_font_sizes)/sizeof(int) );
     for ( int i=0; i<sizes.length(); i++ )
@@ -216,9 +291,10 @@ SettingsDlg::SettingsDlg(QWidget *parent, CR3View * docView ) :
     m_ui->cbTextFontSize->addItems( sizeList );
     m_ui->cbTitleFontSize->addItems( sizeList );
     
-    const char * defFontFace = "DejaVu Sans";
+    m_defFontFace = "DejaVu Sans";
     static const char * goodFonts[] = {
         "DejaVu Sans",
+        "Noto Sans",
         "FreeSans",
         "Liberation Sans",
         "Arial",
@@ -226,14 +302,25 @@ SettingsDlg::SettingsDlg(QWidget *parent, CR3View * docView ) :
     };
     for ( int i=0; goodFonts[i]; i++ ) {
         if ( m_faceList.indexOf(QString(goodFonts[i]))>=0 ) {
-            defFontFace = goodFonts[i];
+            m_defFontFace = goodFonts[i];
             break;
         }
     }
 
-    fontToUi( PROP_FONT_FACE, PROP_FONT_SIZE, m_ui->cbTextFontFace, m_ui->cbTextFontSize, defFontFace );
-    fontToUi( PROP_STATUS_FONT_FACE, PROP_STATUS_FONT_SIZE, m_ui->cbTitleFontFace, m_ui->cbTitleFontSize, defFontFace );
-    fontToUi( PROP_FALLBACK_FONT_FACE, PROP_FALLBACK_FONT_FACE, m_ui->cbFallbackFontFace, NULL, defFontFace );
+    fontToUi( PROP_FONT_FACE, PROP_FONT_SIZE, m_ui->cbTextFontFace, m_ui->cbTextFontSize, m_defFontFace.toLatin1().data() );
+    fontToUi( PROP_STATUS_FONT_FACE, PROP_STATUS_FONT_SIZE, m_ui->cbTitleFontFace, m_ui->cbTitleFontSize, m_defFontFace.toLatin1().data() );
+
+    updateFontWeights();
+
+    int aa = m_props->getIntDef( PROP_FONT_ANTIALIASING, (int)font_aa_gray );
+    int aai = 1;
+    for (int i=0; i < MAX_AA_INDEX; i++) {
+        if (aa == aa_variants[i]) {
+            aai = i;
+            break;
+        }
+    }
+    m_ui->cbAntialiasingMode->setCurrentIndex( aai );
 
 //		{_("90%"), "90"},
 //		{_("100%"), "100"},
@@ -250,6 +337,8 @@ SettingsDlg::SettingsDlg(QWidget *parent, CR3View * docView ) :
     int isi = m_ui->cbInterlineSpace->findText(v);
     m_ui->cbInterlineSpace->setCurrentIndex(isi>=0 ? isi : 6);
 
+    optionToUi( PROP_TEXTLANG_EMBEDDED_LANGS_ENABLED, m_ui->cbMultiLang );
+    optionToUi( PROP_TEXTLANG_HYPHENATION_ENABLED, m_ui->cbEnableHyph );
     int hi = -1;
     v = m_props->getStringDef(PROP_HYPHENATION_DICT,"@algorithm"); //HYPH_DICT_ID_ALGORITHM;
     for ( int i=0; i<HyphMan::getDictList()->length(); i++ ) {
@@ -264,13 +353,28 @@ SettingsDlg::SettingsDlg(QWidget *parent, CR3View * docView ) :
         m_ui->cbHyphenation->addItem( s );
     }
     m_ui->cbHyphenation->setCurrentIndex(hi>=0 ? hi : 1);
+    bool legacy_render = (0 == rendFlags) || (0 == DOMVersionIndex);
+    m_ui->label_47->setVisible(!legacy_render);
+    m_ui->cbMultiLang->setVisible(!legacy_render);
+    m_ui->label_48->setVisible(!legacy_render);
+    m_ui->cbEnableHyph->setVisible(!legacy_render);
+    if (legacy_render) {
+        m_ui->label_9->setVisible(true);
+        m_ui->cbHyphenation->setVisible(true);
+    } else {
+        bool embedded_lang = m_props->getBoolDef(PROP_TEXTLANG_EMBEDDED_LANGS_ENABLED, true);
+        m_ui->label_9->setVisible(!embedded_lang);
+        m_ui->cbHyphenation->setVisible(!embedded_lang);
+        m_ui->label_48->setVisible(embedded_lang);
+        m_ui->cbEnableHyph->setVisible(embedded_lang);
+    }
 
 
     m_ui->crSample->setOptions( m_props );
     m_ui->crSample->getDocView()->setShowCover( false );
     m_ui->crSample->getDocView()->setViewMode( DVM_SCROLL, 1 );
     QString testPhrase = tr("The quick brown fox jumps over the lazy dog. ");
-    m_ui->crSample->getDocView()->createDefaultDocument(lString16::empty_str, qt2cr(testPhrase+testPhrase+testPhrase));
+    m_ui->crSample->getDocView()->createDefaultDocument(lString32::empty_str, qt2cr(testPhrase+testPhrase+testPhrase));
 
     updateStyleSample();
 
@@ -306,6 +410,7 @@ SettingsDlg::~SettingsDlg()
 bool SettingsDlg::showDlg(  QWidget * parent, CR3View * docView )
 {
     SettingsDlg * dlg = new SettingsDlg( parent, docView );
+    dlg->setAttribute(Qt::WA_DeleteOnClose, true);
     dlg->show();
     return true;
 }
@@ -329,23 +434,20 @@ void SettingsDlg::on_buttonBox_rejected()
 void SettingsDlg::on_cbFontKerning_stateChanged(int s)
 {
     setCheck( PROP_FONT_KERNING_ENABLED, s );
-}
-
-void SettingsDlg::on_cbLigatures_stateChanged(int s)
-{
-    setCheck( PROP_FONT_LIGATURES_ENABLED, s );
+    updateStyleSample();
 }
 
 void SettingsDlg::on_buttonBox_accepted()
 {
     m_docview->setOptions( m_props );
+    m_docview->getDocView()->requestRender();
     close();
 }
 
 void SettingsDlg::optionToUi( const char * optionName, QCheckBox * cb )
 {
-    int state = ( m_props->getIntDef( optionName, 1 ) != 0 ) ? 1 : 0;
-    CRLog::debug("optionToUI(%s,%d)", optionName, state);
+    bool state = m_props->getBoolDef( optionName, true );
+    CRLog::debug("optionToUI(%s,%d)", optionName, (int)state);
     cb->setCheckState( state ? Qt::Checked : Qt::Unchecked );
 }
 
@@ -741,10 +843,10 @@ void SettingsDlg::on_cbViewMode_currentIndexChanged(int index)
     }
 }
 
-void SettingsDlg::on_cbShowPageHeader_stateChanged(int s)
+void SettingsDlg::on_cbShowPageHeader_currentIndexChanged(int index)
 {
-    setCheckInversed( PROP_STATUS_LINE, s );
-    bool b = m_props->getIntDef( PROP_STATUS_LINE, 0 )==0;
+    m_props->setInt( PROP_STATUS_LINE, index );
+    bool b = index > 0;
     m_ui->cbShowBattery->setEnabled( b );
     m_ui->cbShowClock->setEnabled( b );
     m_ui->cbShowBookName->setEnabled( b );
@@ -809,7 +911,8 @@ void SettingsDlg::updateStyleSample()
 
     m_ui->crSample->getDocView()->getPageImage(0);
 
-    HyphMan::getDictList()->activate( qt2cr(m_oldHyph) );
+    if (!m_props->getBoolDef(PROP_TEXTLANG_EMBEDDED_LANGS_ENABLED, true))
+        HyphMan::getDictList()->activate( qt2cr(m_oldHyph) );
 }
 
 QColor SettingsDlg::getColor( const char * optionName, unsigned def )
@@ -876,6 +979,7 @@ void SettingsDlg::on_cbTextFontFace_currentIndexChanged(QString s)
     if ( !initDone )
         return;
     m_props->setString( PROP_FONT_FACE, s );
+    updateFontWeights();
     updateStyleSample();
 }
 
@@ -901,6 +1005,76 @@ void SettingsDlg::fontToUi( const char * faceOptionName, const char * sizeOption
     }
 }
 
+void SettingsDlg::updateFontWeights() {
+    QString selectedFontFace = m_ui->cbTextFontFace->currentText();
+    lString8 face8 = UnicodeToUtf8(qt2cr(selectedFontFace));
+    int i;
+    int weight = 0;
+    LVArray<int> weights;
+    LVArray<int> nativeWeights;
+    fontMan->GetAvailableFontWeights(nativeWeights, face8);
+    if (nativeWeights.length() > 0) {
+        // combine with synthetic weights
+        int synth_idx = 0;
+        int j;
+        int prev_weight = 0;
+        for (i = 0; i < nativeWeights.length(); i++) {
+            weight = nativeWeights[i];
+            for (j = synth_idx; j < SYNTH_WEIGHTS_SZ; j++) {
+                int synth_weight = synth_weights[j];
+                if (synth_weight < weight) {
+                    if (synth_weight > prev_weight)
+                        weights.add(synth_weight);
+                }
+                else
+                    break;
+            }
+            synth_idx = j;
+            weights.add(weight);
+            prev_weight = weight;
+        }
+        for (j = synth_idx; j < SYNTH_WEIGHTS_SZ; j++) {
+            if (synth_weights[j] > weight)
+                weights.add(synth_weights[j]);
+        }
+    }
+    // fill items
+    suppressOnChange = true;
+    int normalIndex = 0;
+    m_ui->cbFontWeightChange->clear();
+    if (weights.empty()) {
+        // Invalid font
+        return;
+    }
+    for (i = 0; i < weights.length(); i++) {
+        weight = weights[i];
+        if (400 == weight)
+            normalIndex = i;
+        QString label = QString("%1").arg(weight);
+        QString descr = getWeightName(weight);
+        if (nativeWeights.indexOf(weight) < 0) {
+            if (!descr.isEmpty())
+                descr += QString(", %1").arg(tr("synthetic*"));
+            else
+                descr = tr("synthetic");
+        }
+        if (!descr.isEmpty())
+            label = QString("%1 (%2)").arg(label).arg(descr);
+        m_ui->cbFontWeightChange->addItem(label, QVariant(weight));
+    }
+    // select active
+    int fontWeightIndex = normalIndex;
+    int fontWeight = m_props->getIntDef(PROP_FONT_BASE_WEIGHT, 400);
+    for (i = 0; i < weights.length(); i++) {
+        if (fontWeight == weights[i]) {
+            fontWeightIndex = i;
+            break;
+        }
+    }
+    suppressOnChange = false;
+    m_ui->cbFontWeightChange->setCurrentIndex(fontWeightIndex);
+}
+
 void SettingsDlg::on_cbInterlineSpace_currentIndexChanged(int index)
 {
     if ( !initDone )
@@ -944,11 +1118,13 @@ void SettingsDlg::on_cbPageSkin_currentIndexChanged(int index)
 void SettingsDlg::on_cbFloatingPunctuation_stateChanged(int s)
 {
     setCheck( PROP_FLOATING_PUNCTUATION, s );
+    updateStyleSample();
 }
 
 void SettingsDlg::on_cbFontGamma_currentIndexChanged(QString s)
 {
     m_props->setString( PROP_FONT_GAMMA, s );
+    updateStyleSample();
 }
 
 void SettingsDlg::on_cbStyleName_currentIndexChanged(int index)
@@ -1015,13 +1191,6 @@ void SettingsDlg::on_cbDefFontColor_currentIndexChanged(int index)
 void SettingsDlg::on_cbFontHinting_currentIndexChanged(int index)
 {
     m_props->setInt(PROP_FONT_HINTING, index);
-}
-
-void SettingsDlg::on_cbFallbackFontFace_currentIndexChanged(const QString &s)
-{
-    if ( !initDone )
-        return;
-    m_props->setString(PROP_FALLBACK_FONT_FACE, s);
     updateStyleSample();
 }
 
@@ -1030,10 +1199,17 @@ void SettingsDlg::on_cbEnableEmbeddedFonts_toggled(bool checked)
     setCheck(PROP_EMBEDDED_FONTS, checked ? Qt::Checked : Qt::Unchecked);
 }
 
+void SettingsDlg::on_cbIgnoreDocumentMargins_stateChanged(int s)
+{
+    QString value = (s == Qt::Checked) ? "margin: 0em !important" : "";
+    m_props->setString("styles.body.margin", value);
+}
+
 void SettingsDlg::on_cbEnableDocumentStyles_toggled(bool checked)
 {
     setCheck(PROP_EMBEDDED_STYLES, checked ? Qt::Checked : Qt::Unchecked);
     m_ui->cbEnableEmbeddedFonts->setEnabled(checked);
+    m_ui->cbIgnoreDocumentMargins->setEnabled(checked);
 }
 
 void SettingsDlg::on_btnSelectionColor_clicked()
@@ -1097,3 +1273,108 @@ void SettingsDlg::on_cbImageBlockZoomoutScale_currentIndexChanged(int index)
 {
     m_props->setInt(PROP_IMG_SCALING_ZOOMOUT_BLOCK_SCALE, index);
 }
+
+void SettingsDlg::on_cbFontShaping_currentIndexChanged(int index)
+{
+    m_props->setInt(PROP_FONT_SHAPING, index);
+    updateStyleSample();
+}
+
+void SettingsDlg::on_cbRendFlags_currentIndexChanged(int index)
+{
+    if (index < 0 || index >= MAX_REND_FLAGS_INDEX)
+        index = 0;
+    m_props->setInt(PROP_RENDER_BLOCK_RENDERING_FLAGS, rend_flags[index]);
+    int DOMVersion = m_props->getIntDef(PROP_REQUESTED_DOM_VERSION, gDOMVersionCurrent);
+    bool legacy_render = (0 == index) || (DOMVersion < 20180524);
+    m_ui->label_47->setVisible(!legacy_render);
+    m_ui->cbMultiLang->setVisible(!legacy_render);
+    m_ui->label_48->setVisible(!legacy_render);
+    m_ui->cbEnableHyph->setVisible(!legacy_render);
+    if (legacy_render) {
+        m_ui->label_9->setVisible(true);
+        m_ui->cbHyphenation->setVisible(true);
+    } else {
+        bool embedded_lang = m_props->getBoolDef(PROP_TEXTLANG_EMBEDDED_LANGS_ENABLED, true);
+        m_ui->label_9->setVisible(!embedded_lang);
+        m_ui->cbHyphenation->setVisible(!embedded_lang);
+        m_ui->label_48->setVisible(embedded_lang);
+        m_ui->cbEnableHyph->setVisible(embedded_lang);
+    }
+    updateStyleSample();
+}
+
+void SettingsDlg::on_cbDOMLevel_currentIndexChanged(int index)
+{
+    if (index < 0 || index >= MAX_DOM_VERSIONS_INDEX)
+        index = 0;
+    int rendFlags = m_props->getIntDef(PROP_RENDER_BLOCK_RENDERING_FLAGS, 0);
+    bool legacy_render = (0 == rendFlags) || (0 == index);
+    m_ui->label_47->setVisible(!legacy_render);
+    m_ui->cbMultiLang->setVisible(!legacy_render);
+    m_ui->label_48->setVisible(!legacy_render);
+    m_ui->cbEnableHyph->setVisible(!legacy_render);
+    if (legacy_render) {
+        m_ui->label_9->setVisible(true);
+        m_ui->cbHyphenation->setVisible(true);
+    } else {
+        bool embedded_lang = m_props->getBoolDef(PROP_TEXTLANG_EMBEDDED_LANGS_ENABLED, true);
+        m_ui->label_9->setVisible(!embedded_lang);
+        m_ui->cbHyphenation->setVisible(!embedded_lang);
+        m_ui->label_48->setVisible(embedded_lang);
+        m_ui->cbEnableHyph->setVisible(embedded_lang);
+    }
+    m_props->setInt(PROP_REQUESTED_DOM_VERSION, DOM_versions[index]);
+    updateStyleSample();
+}
+
+void SettingsDlg::on_cbMultiLang_stateChanged(int state)
+{
+    setCheck( PROP_TEXTLANG_EMBEDDED_LANGS_ENABLED, state );
+    m_ui->label_9->setVisible(state != Qt::Checked);
+    m_ui->cbHyphenation->setVisible(state != Qt::Checked);
+    m_ui->label_48->setVisible(state == Qt::Checked);
+    m_ui->cbEnableHyph->setVisible(state == Qt::Checked);
+    // don't update preview to not change static field TextLangMan::_embedded_langs_enabled too early!
+}
+
+void SettingsDlg::on_cbEnableHyph_stateChanged(int state)
+{
+    setCheck( PROP_TEXTLANG_HYPHENATION_ENABLED, state );
+    // don't update preview to not change static field TextLangMan::_hyphenation_enabled too early!
+}
+
+void SettingsDlg::on_btnFallbackMan_clicked()
+{
+    FallbackFontsDialog dlg(this, m_faceList);
+    QString fallbackFaces = m_props->getStringDef( PROP_FALLBACK_FONT_FACES, m_defFontFace.toLatin1().data() );
+    dlg.setFallbackFaces(fallbackFaces);
+    if ( dlg.exec() == QDialog::Accepted ) {
+        fallbackFaces = dlg.fallbackFaces();
+        m_props->setString( PROP_FALLBACK_FONT_FACES, fallbackFaces );
+        updateStyleSample();
+    }
+}
+
+void SettingsDlg::on_cbFontWeightChange_currentIndexChanged(int index)
+{
+    if (suppressOnChange)
+        return;
+    QVariant data = m_ui->cbFontWeightChange->currentData(Qt::UserRole);
+    int weight = data.toInt();
+    m_props->setInt(PROP_FONT_BASE_WEIGHT, weight);
+    QString face = m_props->getStringDef( PROP_FONT_FACE, "" );
+    LVArray<int> nativeWeights;
+    fontMan->GetAvailableFontWeights(nativeWeights, UnicodeToUtf8(qt2cr(face)));
+    //m_ui->cbFontHinting->setEnabled(nativeWeights.indexOf(weight) >= 0);
+    updateStyleSample();
+}
+
+void SettingsDlg::on_cbAntialiasingMode_currentIndexChanged(int index)
+{
+    if (index < 0 || index >= MAX_AA_INDEX)
+        index = 0;
+    int aa = aa_variants[index];
+    m_props->setInt( PROP_FONT_ANTIALIASING, aa );
+    updateStyleSample();
+}
